# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: onyx_otc/v2/common.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'onyx_otc/v2/common.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18onyx_otc/v2/common.proto\x12\x0bonyx_otc.v2\"\x18\n\x07\x44\x65\x63imal\x12\r\n\x05value\x18\x01 \x01(\t\"%\n\x06Spread\x12\r\n\x05\x66ront\x18\x01 \x01(\t\x12\x0c\n\x04\x62\x61\x63k\x18\x02 \x01(\t\"8\n\tButterfly\x12\r\n\x05\x66ront\x18\x01 \x01(\t\x12\x0e\n\x06middle\x18\x02 \x01(\t\x12\x0c\n\x04\x62\x61\x63k\x18\x03 \x01(\t\"~\n\x0eTradableSymbol\x12\x0e\n\x04\x66lat\x18\x01 \x01(\tH\x00\x12%\n\x06spread\x18\x02 \x01(\x0b\x32\x13.onyx_otc.v2.SpreadH\x00\x12+\n\tbutterfly\x18\x03 \x01(\x0b\x32\x16.onyx_otc.v2.ButterflyH\x00\x42\x08\n\x06symbolB\x0b\xaa\x02\x08Onyx.Otcb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'onyx_otc.v2.common_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\252\002\010Onyx.Otc'
  _globals['_DECIMAL']._serialized_start=41
  _globals['_DECIMAL']._serialized_end=65
  _globals['_SPREAD']._serialized_start=67
  _globals['_SPREAD']._serialized_end=104
  _globals['_BUTTERFLY']._serialized_start=106
  _globals['_BUTTERFLY']._serialized_end=162
  _globals['_TRADABLESYMBOL']._serialized_start=164
  _globals['_TRADABLESYMBOL']._serialized_end=290
# @@protoc_insertion_point(module_scope)



import asyncio
import logging
from dataclasses import dataclass, field

import click

from onyx_otc.requests import InvalidInputError, RfqChannel
from onyx_otc.responses import OtcChannelMessage, OtcResponse
from onyx_otc.websocket_v2 import OnyxWebsocketClientV2

from . import common

logger = logging.getLogger(__name__)


@dataclass
class Workflow:
    server_info: bool = False
    tickers: list[str] = field(default_factory=list)
    obt: list[str] = field(default_factory=list)
    rfqs: list[RfqChannel] = field(default_factory=list)

    def on_response(self, cli: OnyxWebsocketClientV2, response: OtcResponse) -> None:
        response.log()
        if response.auth():
            if self.server_info:
                cli.subscribe_server_info()
            if self.tickers:
                cli.subscribe_tickers(self.tickers)
            if self.obt:
                cli.subscribe_obt(self.obt)
            for rfq in self.rfqs:
                cli.subscribe_rfq(rfq)

    def on_event(self, cli: OnyxWebsocketClientV2, message: OtcChannelMessage) -> None:
        message.log()


async def run_client_websocket(
    workflow: Workflow,
    binary: bool,
    token: str | None = None,
    ws_url: str | None = None,
) -> None:
    client = OnyxWebsocketClientV2.create(
        binary=binary,
        on_response=workflow.on_response,
        on_event=workflow.on_event,
        api_token=token,
        ws_url=ws_url,
    )
    await client.connect()


@click.command()
@click.option(
    "--tickers",
    "-t",
    multiple=True,
    help="Product symbols to subscribe to tickers channel",
)
@click.option(
    "--obt",
    "-o",
    multiple=True,
    help="Product symbols to subscribe to order-book-top channel",
)
@click.option(
    "--server-info",
    "-s",
    is_flag=True,
    help="Subscribe to server info",
)
@click.option(
    "--rfq",
    "-r",
    help="RFQ symbols as <symbol>@<exchange>@<size=1>",
    multiple=True,
)
@click.option(
    "--json",
    "-j",
    is_flag=True,
    help="Use JSON stream instead of protobuf",
)
@common.token_option
@common.url_option
def stream(
    tickers: list[str],
    obt: list[str],
    server_info: bool,
    rfq: list[str],
    json: bool,
    token: str | None,
    url: str | None,
) -> None:
    """Stream websocket data from Onyx."""
    try:
        workflow = Workflow(
            server_info=server_info,
            tickers=tickers,
            obt=obt,
            rfqs=[RfqChannel.from_string(r) for r in rfq],
        )
    except InvalidInputError as e:
        click.echo(e, err=True)
        raise click.Abort() from None
    try:
        asyncio.run(
            run_client_websocket(workflow, binary=not json, token=token, ws_url=url)
        )
    except KeyboardInterrupt:
        pass



import logging

import click
import dotenv

import onyx_otc

from .stream import stream

dotenv.load_dotenv()


@click.group()
@click.version_option(onyx_otc.__version__)
@click.option(
    "--debug",
    is_flag=True,
    help="Enable debug logging",
)
def cli(debug: bool) -> None:
    """Command line interface for onyx-otc."""
    level = logging.DEBUG if debug else logging.INFO
    logging.basicConfig(level=level, format="%(message)s")


cli.add_command(stream)


if __name__ == "__main__":
    cli()



from __future__ import annotations

from decimal import Decimal
from typing import Any, Self

from pydantic import BaseModel

from .common import TradableSymbol
from .timestamp import Timestamp
from .types import Channel, Exchange, Method, OrderType, Side
from .v2 import common_pb2, requests_pb2


class InvalidInputError(ValueError):
    pass


class AuthRequest(BaseModel):
    """Request for authentication."""

    token: str

    def to_proto(self) -> requests_pb2.Auth:
        return requests_pb2.Auth(token=self.token)


class OtcOrderRequest(BaseModel):
    """Request for placing an order."""

    account_id: str
    symbol: TradableSymbol
    quantity: Decimal
    side: Side
    price: Decimal
    order_type: OrderType = OrderType.FILL_OR_KILL
    client_order_id: str = ""

    def to_proto(self) -> requests_pb2.NewOrderRequest:
        return requests_pb2.NewOrderRequest(
            account_id=self.account_id,
            symbol=self.symbol.to_proto(),
            quantity=common_pb2.Decimal(value=str(self.quantity)),
            side=self.side.to_proto(),
            price=common_pb2.Decimal(value=str(self.price)),
            order_type=self.order_type.to_proto(),
            client_order_id=self.client_order_id,
        )


class TickersChannel(BaseModel):
    """Request for subscribing to ticker updates for a list of product symbols."""

    products: list[str]

    def to_proto(self) -> requests_pb2.TickersChannel:
        return requests_pb2.TickersChannel(products=self.products)


class OrderBookTopChannel(BaseModel):
    """Request for subscribing to order book top updates for
    a list of product symbols.
    """

    products: list[str]

    def to_proto(self) -> requests_pb2.OrderBookTopChannel:
        return requests_pb2.OrderBookTopChannel(products=self.products)


class ServerInfoChannel(BaseModel):

    def to_proto(self) -> requests_pb2.ServerInfoChannel:
        return requests_pb2.ServerInfoChannel()


class OrdersChannel(BaseModel):

    def to_proto(self) -> requests_pb2.OrdersChannel:
        return requests_pb2.OrdersChannel()


class RfqChannel(BaseModel):
    """Request for subscribing to RFQ updates for a symbol."""

    symbol: TradableSymbol
    exchange: Exchange
    size: Decimal = Decimal(1)

    @classmethod
    def from_string(cls, rfq: str) -> Self:
        bits = rfq.split("@")
        if len(bits) > 1:
            symbol = TradableSymbol.from_string(bits[0])
            exchange = Exchange[bits[1].upper()]
            size = Decimal(bits[2]) if len(bits) > 2 else Decimal(1)
            return cls(symbol=symbol, exchange=exchange, size=size)
        else:
            raise InvalidInputError(
                f"Invalid RFQ format: {rfq}. Expected <symbol>@<exchange>@<size>"
            )

    def to_proto(self) -> requests_pb2.RfqChannel:
        return requests_pb2.RfqChannel(
            symbol=self.symbol.to_proto(),
            size=common_pb2.Decimal(value=str(self.size)),
            exchange=self.exchange.to_proto(),
        )

    def model_dump(self, **kwargs: Any) -> dict:
        """Customize serialization to ensure `symbol` is a string."""
        data = super().model_dump(**kwargs)
        data["symbol"] = self.symbol.as_string()  # Convert symbol to string
        return data


channel_from_data_type = {
    ServerInfoChannel: Channel.SERVER_INFO,
    TickersChannel: Channel.TICKERS,
    OrdersChannel: Channel.ORDERS,
    OrderBookTopChannel: Channel.ORDER_BOOK_TOP,
    RfqChannel: Channel.RFQ,
}


class SubscribeRequestBase(BaseModel):
    data: (
        ServerInfoChannel
        | TickersChannel
        | OrdersChannel
        | RfqChannel
        | OrderBookTopChannel
    )

    @property
    def channel(self) -> Channel:
        return channel_from_data_type[type(self.data)]

    def proto_dict(self) -> dict:
        return {self.channel.subscribe_proto_key: self.data.to_proto()}

    def model_dump(self, **kwargs: Any) -> dict:
        kwargs["mode"] = "json"
        data = self.data.model_dump(**kwargs)
        return {"channel": {self.channel.value: data}}


class SubscribeRequest(SubscribeRequestBase):
    """Request for subscribing to a channel."""

    def to_proto(self) -> requests_pb2.Subscribe:
        return requests_pb2.Subscribe(**self.proto_dict())  # type: ignore[arg-type]


class UnsubscribeRequest(SubscribeRequestBase):
    """Request for unsubscribing from a channel."""

    def to_proto(self) -> requests_pb2.Unsubscribe:
        return requests_pb2.Unsubscribe(**self.proto_dict())  # type: ignore[arg-type]


request_method_from_data_type = {
    AuthRequest: Method.AUTH,
    OtcOrderRequest: Method.ORDER,
    SubscribeRequest: Method.SUBSCRIBE,
    UnsubscribeRequest: Method.UNSUBSCRIBE,
}


class OtcRequest(BaseModel):
    """A request to the websocket API."""

    id: str
    timestamp: Timestamp
    request: AuthRequest | OtcOrderRequest | SubscribeRequest | UnsubscribeRequest

    @property
    def method(self) -> Method:
        return request_method_from_data_type[type(self.request)]

    def to_proto(self) -> requests_pb2.OtcRequest:
        method = self.method
        return requests_pb2.OtcRequest(
            id=self.id,
            timestamp=self.timestamp.to_proto(),
            method=method.to_proto(),
            **{method.value: self.request.to_proto()},  # type: ignore[arg-type]
        )

    def to_json_dict(self) -> dict:
        return dict(
            id=self.id,
            method=self.method.value,
            timestamp=self.timestamp.to_datetime().isoformat(),
            **self.request.model_dump(),
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: onyx_otc/v2/types.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'onyx_otc/v2/types.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17onyx_otc/v2/types.proto\x12\x0bonyx_otc.v2*\xab\x02\n\x0cOtcErrorCode\x12\x1e\n\x1aOTC_ERROR_CODE_UNSPECIFIED\x10\x00\x12\"\n\x1eOTC_ERROR_CODE_INVALID_REQUEST\x10\x01\x12\"\n\x1eOTC_ERROR_CODE_NOT_IMPLEMENTED\x10\x02\x12\"\n\x1eOTC_ERROR_CODE_UNAUTHENTICATED\x10\x03\x12$\n OTC_ERROR_CODE_TOO_MANY_REQUESTS\x10\x04\x12!\n\x1dOTC_ERROR_CODE_NOT_SUBSCRIBED\x10\x05\x12\x1c\n\x18OTC_ERROR_CODE_FORBIDDEN\x10\x06\x12(\n$OTC_ERROR_CODE_INTERNAL_SERVER_ERROR\x10\x1e*9\n\x04Side\x12\x14\n\x10SIDE_UNSPECIFIED\x10\x00\x12\x0c\n\x08SIDE_BUY\x10\x01\x12\r\n\tSIDE_SELL\x10\x02*\x83\x01\n\x12SubscriptionStatus\x12#\n\x1fSUBSCRIPTION_STATUS_UNSPECIFIED\x10\x00\x12\"\n\x1eSUBSCRIPTION_STATUS_SUBSCRIBED\x10\x01\x12$\n SUBSCRIPTION_STATUS_UNSUBSCRIBED\x10\x02*q\n\x06Method\x12\x16\n\x12METHOD_UNSPECIFIED\x10\x00\x12\x0f\n\x0bMETHOD_AUTH\x10\x01\x12\x10\n\x0cMETHOD_ORDER\x10\x02\x12\x14\n\x10METHOD_SUBSCRIBE\x10\x03\x12\x16\n\x12METHOD_UNSUBSCRIBE\x10\x04*H\n\x08\x45xchange\x12\x18\n\x14\x45XCHANGE_UNSPECIFIED\x10\x00\x12\x10\n\x0c\x45XCHANGE_ICE\x10\x01\x12\x10\n\x0c\x45XCHANGE_CME\x10\x02*D\n\tOrderType\x12\x1a\n\x16ORDER_TYPE_UNSPECIFIED\x10\x00\x12\x1b\n\x17ORDER_TYPE_FILL_OR_KILL\x10\x01*\xa9\x01\n\nOrderState\x12\x1b\n\x17ORDER_STATE_UNSPECIFIED\x10\x00\x12\x1f\n\x1bORDER_STATE_PENDING_CREATED\x10\x01\x12\x17\n\x13ORDER_STATE_CREATED\x10\x02\x12\x13\n\x0fORDER_STATE_NEW\x10\x03\x12\x16\n\x12ORDER_STATE_FILLED\x10\x06\x12\x17\n\x13ORDER_STATE_ERRORED\x10\n*\xcd\x03\n\x0cTradingError\x12\x1d\n\x19TRADING_ERROR_UNSPECIFIED\x10\x00\x12\x1c\n\x17TRADING_ERROR_FORBIDDEN\x10\x93\x03\x12\x1c\n\x17TRADING_ERROR_NOT_FOUND\x10\x94\x03\x12#\n\x1eTRADING_ERROR_TRADING_DISABLED\x10\x95\x03\x12$\n\x1fTRADING_ERROR_TRADING_SUSPENDED\x10\x96\x03\x12!\n\x1cTRADING_ERROR_NOT_SUBSCRIBED\x10\x97\x03\x12!\n\x1cTRADING_ERROR_INVALID_SYMBOL\x10\x9d\x03\x12 \n\x1bTRADING_ERROR_INVALID_VALUE\x10\xa6\x03\x12 \n\x1bTRADING_ERROR_INVALID_STATE\x10\xa8\x03\x12 \n\x1bTRADING_ERROR_INVALID_PRICE\x10\xa9\x03\x12$\n\x1fTRADING_ERROR_CONTRACT_EXPIRING\x10\xaa\x03\x12!\n\x1cTRADING_ERROR_INTERNAL_ERROR\x10\xf4\x03\x12\"\n\x1dTRADING_ERROR_NOT_IMPLEMENTED\x10\xf5\x03*\x91\x01\n\x07\x43hannel\x12\x17\n\x13\x43HANNEL_UNSPECIFIED\x10\x00\x12\x12\n\x0e\x43HANNEL_ORDERS\x10\x01\x12\x17\n\x13\x43HANNEL_SERVER_INFO\x10\x02\x12\x13\n\x0f\x43HANNEL_TICKERS\x10\x03\x12\x0f\n\x0b\x43HANNEL_RFQ\x10\x04\x12\x1a\n\x16\x43HANNEL_ORDER_BOOK_TOP\x10\x05\x42\x0b\xaa\x02\x08Onyx.Otcb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'onyx_otc.v2.types_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\252\002\010Onyx.Otc'
  _globals['_OTCERRORCODE']._serialized_start=41
  _globals['_OTCERRORCODE']._serialized_end=340
  _globals['_SIDE']._serialized_start=342
  _globals['_SIDE']._serialized_end=399
  _globals['_SUBSCRIPTIONSTATUS']._serialized_start=402
  _globals['_SUBSCRIPTIONSTATUS']._serialized_end=533
  _globals['_METHOD']._serialized_start=535
  _globals['_METHOD']._serialized_end=648
  _globals['_EXCHANGE']._serialized_start=650
  _globals['_EXCHANGE']._serialized_end=722
  _globals['_ORDERTYPE']._serialized_start=724
  _globals['_ORDERTYPE']._serialized_end=792
  _globals['_ORDERSTATE']._serialized_start=795
  _globals['_ORDERSTATE']._serialized_end=964
  _globals['_TRADINGERROR']._serialized_start=967
  _globals['_TRADINGERROR']._serialized_end=1428
  _globals['_CHANNEL']._serialized_start=1431
  _globals['_CHANNEL']._serialized_end=1576
# @@protoc_insertion_point(module_scope)



import click

token_option = click.option(
    "--token",
    help=(
        "API token - if not provided, will be read from "
        "ONYX_API_TOKEN environment variable and the .env file"
    ),
    type=str,
)


url_option = click.option(
    "--url",
    help=(
        "Websocket API url - if not provided, will be read from "
        "ONYX_WS_V2_URL environment variable and the .env file"
    ),
    type=str,
)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: onyx_otc/v2/responses.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'onyx_otc/v2/responses.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from onyx_otc.v2 import common_pb2 as onyx__otc_dot_v2_dot_common__pb2
from onyx_otc.v2 import types_pb2 as onyx__otc_dot_v2_dot_types__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1bonyx_otc/v2/responses.proto\x12\x0bonyx_otc.v2\x1a\x18onyx_otc/v2/common.proto\x1a\x17onyx_otc/v2/types.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"L\n\x06Trader\x12\n\n\x02id\x18\x01 \x01(\t\x12\x14\n\x0c\x61\x63\x63ount_name\x18\x02 \x01(\t\x12\x11\n\tfull_name\x18\x03 \x01(\t\x12\r\n\x05\x65mail\x18\x04 \x01(\t\"X\n\x0bPriceAmount\x12#\n\x05price\x18\x01 \x01(\x0b\x32\x14.onyx_otc.v2.Decimal\x12$\n\x06\x61mount\x18\x02 \x01(\x0b\x32\x14.onyx_otc.v2.Decimal\"\xd9\x01\n\x06Ticker\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12-\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12!\n\x03mid\x18\x03 \x01(\x0b\x32\x14.onyx_otc.v2.Decimal\x12(\n\nfull_price\x18\x04 \x01(\x0b\x32\x14.onyx_otc.v2.Decimal\x12+\n\raccrued_price\x18\x05 \x01(\x0b\x32\x14.onyx_otc.v2.Decimal\x12\x16\n\x0eproduct_symbol\x18\x06 \x01(\t\"h\n\x03\x46\x65\x65\x12\x10\n\x08\x63urrency\x18\x01 \x01(\t\x12+\n\rsigned_amount\x18\x02 \x01(\x0b\x32\x14.onyx_otc.v2.Decimal\x12\"\n\x04rate\x18\x03 \x01(\x0b\x32\x14.onyx_otc.v2.Decimal\"\xf4\x02\n\tExecution\x12\n\n\x02id\x18\x01 \x01(\t\x12\x10\n\x08order_id\x18\x02 \x01(\t\x12\x17\n\x0f\x63lient_order_id\x18\x03 \x01(\t\x12\x12\n\naccount_id\x18\x04 \x01(\t\x12\'\n\x08\x65xchange\x18\x05 \x01(\x0e\x32\x15.onyx_otc.v2.Exchange\x12\x0e\n\x06symbol\x18\x06 \x01(\t\x12$\n\x06\x61mount\x18\x07 \x01(\x0b\x32\x14.onyx_otc.v2.Decimal\x12#\n\x05price\x18\x08 \x01(\x0b\x32\x14.onyx_otc.v2.Decimal\x12\x1f\n\x04side\x18\t \x01(\x0e\x32\x11.onyx_otc.v2.Side\x12\x35\n\x11\x63reated_timestamp\x18\r \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x1d\n\x03\x66\x65\x65\x18\x13 \x01(\x0b\x32\x10.onyx_otc.v2.Fee\x12!\n\x04user\x18\x14 \x01(\x0b\x32\x13.onyx_otc.v2.Trader\"/\n\x07Tickers\x12$\n\x07tickers\x18\x01 \x03(\x0b\x32\x13.onyx_otc.v2.Ticker\"\xde\x02\n\rLiveWebsocket\x12\x12\n\nsocket_uid\x18\x01 \x01(\t\x12\x18\n\x10message_received\x18\x02 \x01(\x03\x12\x14\n\x0cmessage_sent\x18\x03 \x01(\x03\x12+\n\x07started\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x30\n\x0c\x63urrent_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x39\n\x15last_message_received\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x35\n\x11last_message_sent\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x13\n\x0b\x63lient_type\x18\x08 \x01(\t\x12\x11\n\tremote_ip\x18\t \x01(\t\x12\x10\n\x08protocol\x18\n \x01(\t\"i\n\nServerInfo\x12\x12\n\nsocket_uid\x18\x01 \x01(\t\x12\x12\n\nage_millis\x18\x02 \x01(\x03\x12\x33\n\x0flive_websockets\x18\x03 \x03(\x0b\x32\x1a.onyx_otc.v2.LiveWebsocket\"Y\n\x0cOtcQuoteSide\x12#\n\x05price\x18\x01 \x01(\x0b\x32\x14.onyx_otc.v2.Decimal\x12$\n\x06\x61mount\x18\x02 \x01(\x0b\x32\x14.onyx_otc.v2.Decimal\"\x9b\x02\n\x08OtcQuote\x12+\n\x06symbol\x18\x01 \x01(\x0b\x32\x1b.onyx_otc.v2.TradableSymbol\x12\'\n\x08\x65xchange\x18\x02 \x01(\x0e\x32\x15.onyx_otc.v2.Exchange\x12&\n\x03\x62uy\x18\x03 \x01(\x0b\x32\x19.onyx_otc.v2.OtcQuoteSide\x12\'\n\x04sell\x18\x04 \x01(\x0b\x32\x19.onyx_otc.v2.OtcQuoteSide\x12!\n\x03mid\x18\x05 \x01(\x0b\x32\x14.onyx_otc.v2.Decimal\x12-\n\ttimestamp\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x16\n\x0eproduct_symbol\x18\x07 \x01(\t\"\xdd\x01\n\x0cOrderBookTop\x12%\n\x03\x62uy\x18\x01 \x01(\x0b\x32\x18.onyx_otc.v2.PriceAmount\x12&\n\x04sell\x18\x02 \x01(\x0b\x32\x18.onyx_otc.v2.PriceAmount\x12\'\n\x08\x65xchange\x18\x03 \x01(\x0e\x32\x15.onyx_otc.v2.Exchange\x12\x0e\n\x06symbol\x18\x04 \x01(\t\x12-\n\ttimestamp\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x16\n\x0eproduct_symbol\x18\x06 \x01(\t\"C\n\rOrderBookTops\x12\x32\n\x0forder_book_tops\x18\x01 \x03(\x0b\x32\x19.onyx_otc.v2.OrderBookTop\"\x9f\x05\n\x05Order\x12\n\n\x02id\x18\x01 \x01(\t\x12\x17\n\x0f\x63lient_order_id\x18\x02 \x01(\t\x12!\n\x04user\x18\x03 \x01(\x0b\x32\x13.onyx_otc.v2.Trader\x12\x12\n\naccount_id\x18\x04 \x01(\t\x12\'\n\x08\x65xchange\x18\x05 \x01(\x0e\x32\x15.onyx_otc.v2.Exchange\x12\x16\n\x0eproduct_symbol\x18\x06 \x01(\t\x12+\n\x06symbol\x18\x07 \x01(\x0b\x32\x1b.onyx_otc.v2.TradableSymbol\x12*\n\norder_type\x18\n \x01(\x0e\x32\x16.onyx_otc.v2.OrderType\x12,\n\x0border_state\x18\x0b \x01(\x0e\x32\x17.onyx_otc.v2.OrderState\x12$\n\x06\x61mount\x18\x0c \x01(\x0b\x32\x14.onyx_otc.v2.Decimal\x12(\n\x05price\x18\r \x01(\x0b\x32\x14.onyx_otc.v2.DecimalH\x00\x88\x01\x01\x12\x1f\n\x04side\x18\x10 \x01(\x0e\x32\x11.onyx_otc.v2.Side\x12\x35\n\x11\x63reated_timestamp\x18\x11 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x35\n\x12\x65xecuted_avg_price\x18\x15 \x01(\x0b\x32\x14.onyx_otc.v2.DecimalH\x01\x88\x01\x01\x12\x32\n\x0f\x65xecuted_amount\x18\x16 \x01(\x0b\x32\x14.onyx_otc.v2.DecimalH\x02\x88\x01\x01\x12*\n\nexecutions\x18\x1a \x03(\x0b\x32\x16.onyx_otc.v2.ExecutionB\x08\n\x06_priceB\x15\n\x13_executed_avg_priceB\x12\n\x10_executed_amount\"\xd2\x02\n\x0e\x43hannelMessage\x12%\n\x07\x63hannel\x18\x01 \x01(\x0e\x32\x14.onyx_otc.v2.Channel\x12-\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12#\n\x05order\x18\x08 \x01(\x0b\x32\x12.onyx_otc.v2.OrderH\x00\x12.\n\x0bserver_info\x18\t \x01(\x0b\x32\x17.onyx_otc.v2.ServerInfoH\x00\x12\'\n\x07tickers\x18\n \x01(\x0b\x32\x14.onyx_otc.v2.TickersH\x00\x12*\n\totc_quote\x18\x0b \x01(\x0b\x32\x15.onyx_otc.v2.OtcQuoteH\x00\x12\x35\n\x0forder_book_tops\x18\x0c \x01(\x0b\x32\x1a.onyx_otc.v2.OrderBookTopsH\x00\x42\t\n\x07message\"w\n\x0cSubscription\x12%\n\x07\x63hannel\x18\x01 \x01(\x0e\x32\x14.onyx_otc.v2.Channel\x12\x0f\n\x07message\x18\x02 \x01(\t\x12/\n\x06status\x18\x03 \x01(\x0e\x32\x1f.onyx_otc.v2.SubscriptionStatus\"\x1f\n\x0c\x41uthResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\"D\n\x08OtcError\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\'\n\x04\x63ode\x18\x02 \x01(\x0e\x32\x19.onyx_otc.v2.OtcErrorCode\"\xff\x01\n\x0bOtcResponse\x12\n\n\x02id\x18\x01 \x01(\t\x12-\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12#\n\x05order\x18\x0b \x01(\x0b\x32\x12.onyx_otc.v2.OrderH\x00\x12\x31\n\x0csubscription\x18\x0c \x01(\x0b\x32\x19.onyx_otc.v2.SubscriptionH\x00\x12)\n\x04\x61uth\x18\x0e \x01(\x0b\x32\x19.onyx_otc.v2.AuthResponseH\x00\x12&\n\x05\x65rror\x18\r \x01(\x0b\x32\x15.onyx_otc.v2.OtcErrorH\x00\x42\n\n\x08response\"\x86\x01\n\x12OtcResponseMessage\x12\x30\n\x0cotc_response\x18\x01 \x01(\x0b\x32\x18.onyx_otc.v2.OtcResponseH\x00\x12\x36\n\x0f\x63hannel_message\x18\x02 \x01(\x0b\x32\x1b.onyx_otc.v2.ChannelMessageH\x00\x42\x06\n\x04\x64\x61taB\x0b\xaa\x02\x08Onyx.Otcb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'onyx_otc.v2.responses_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\252\002\010Onyx.Otc'
  _globals['_TRADER']._serialized_start=128
  _globals['_TRADER']._serialized_end=204
  _globals['_PRICEAMOUNT']._serialized_start=206
  _globals['_PRICEAMOUNT']._serialized_end=294
  _globals['_TICKER']._serialized_start=297
  _globals['_TICKER']._serialized_end=514
  _globals['_FEE']._serialized_start=516
  _globals['_FEE']._serialized_end=620
  _globals['_EXECUTION']._serialized_start=623
  _globals['_EXECUTION']._serialized_end=995
  _globals['_TICKERS']._serialized_start=997
  _globals['_TICKERS']._serialized_end=1044
  _globals['_LIVEWEBSOCKET']._serialized_start=1047
  _globals['_LIVEWEBSOCKET']._serialized_end=1397
  _globals['_SERVERINFO']._serialized_start=1399
  _globals['_SERVERINFO']._serialized_end=1504
  _globals['_OTCQUOTESIDE']._serialized_start=1506
  _globals['_OTCQUOTESIDE']._serialized_end=1595
  _globals['_OTCQUOTE']._serialized_start=1598
  _globals['_OTCQUOTE']._serialized_end=1881
  _globals['_ORDERBOOKTOP']._serialized_start=1884
  _globals['_ORDERBOOKTOP']._serialized_end=2105
  _globals['_ORDERBOOKTOPS']._serialized_start=2107
  _globals['_ORDERBOOKTOPS']._serialized_end=2174
  _globals['_ORDER']._serialized_start=2177
  _globals['_ORDER']._serialized_end=2848
  _globals['_CHANNELMESSAGE']._serialized_start=2851
  _globals['_CHANNELMESSAGE']._serialized_end=3189
  _globals['_SUBSCRIPTION']._serialized_start=3191
  _globals['_SUBSCRIPTION']._serialized_end=3310
  _globals['_AUTHRESPONSE']._serialized_start=3312
  _globals['_AUTHRESPONSE']._serialized_end=3343
  _globals['_OTCERROR']._serialized_start=3345
  _globals['_OTCERROR']._serialized_end=3413
  _globals['_OTCRESPONSE']._serialized_start=3416
  _globals['_OTCRESPONSE']._serialized_end=3671
  _globals['_OTCRESPONSEMESSAGE']._serialized_start=3674
  _globals['_OTCRESPONSEMESSAGE']._serialized_end=3808
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class Decimal(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUE_FIELD_NUMBER: builtins.int
    value: builtins.str
    def __init__(
        self,
        *,
        value: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["value", b"value"]) -> None: ...

global___Decimal = Decimal

@typing.final
class Spread(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FRONT_FIELD_NUMBER: builtins.int
    BACK_FIELD_NUMBER: builtins.int
    front: builtins.str
    back: builtins.str
    def __init__(
        self,
        *,
        front: builtins.str = ...,
        back: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["back", b"back", "front", b"front"]) -> None: ...

global___Spread = Spread

@typing.final
class Butterfly(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FRONT_FIELD_NUMBER: builtins.int
    MIDDLE_FIELD_NUMBER: builtins.int
    BACK_FIELD_NUMBER: builtins.int
    front: builtins.str
    middle: builtins.str
    back: builtins.str
    def __init__(
        self,
        *,
        front: builtins.str = ...,
        middle: builtins.str = ...,
        back: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["back", b"back", "front", b"front", "middle", b"middle"]) -> None: ...

global___Butterfly = Butterfly

@typing.final
class TradableSymbol(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FLAT_FIELD_NUMBER: builtins.int
    SPREAD_FIELD_NUMBER: builtins.int
    BUTTERFLY_FIELD_NUMBER: builtins.int
    flat: builtins.str
    @property
    def spread(self) -> global___Spread: ...
    @property
    def butterfly(self) -> global___Butterfly: ...
    def __init__(
        self,
        *,
        flat: builtins.str = ...,
        spread: global___Spread | None = ...,
        butterfly: global___Butterfly | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["butterfly", b"butterfly", "flat", b"flat", "spread", b"spread", "symbol", b"symbol"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["butterfly", b"butterfly", "flat", b"flat", "spread", b"spread", "symbol", b"symbol"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["symbol", b"symbol"]) -> typing.Literal["flat", "spread", "butterfly"] | None: ...

global___TradableSymbol = TradableSymbol



from __future__ import annotations

import logging
from datetime import timedelta
from decimal import Decimal
from typing import Annotated, Self

from pydantic import BaseModel, BeforeValidator, Field, field_validator

from .common import PriceAmount, TradableSymbol
from .timestamp import NANOS_PER_MICROS, Timestamp
from .types import Channel, Exchange, OtcErrorCode, Side, SubscriptionStatus
from .v2 import responses_pb2

logger = logging.getLogger(__name__)
AnnotatedTimestamp = Annotated[Timestamp, BeforeValidator(Timestamp.from_any)]


class Auth(BaseModel):
    message: str = ""

    @classmethod
    def from_proto(cls, proto: responses_pb2.AuthResponse) -> Self:
        return cls(message=proto.message)


class LiveWebsocket(BaseModel):
    socket_uid: str = ""
    message_received: int = 0
    message_sent: int = 0
    started: AnnotatedTimestamp = Timestamp()
    current_time: AnnotatedTimestamp = Timestamp()
    last_message_received: AnnotatedTimestamp = Timestamp()
    last_message_sent: AnnotatedTimestamp = Timestamp()
    client_type: str = ""
    remote_ip: str = ""
    protocol: str = ""

    @classmethod
    def from_proto(cls, proto: responses_pb2.LiveWebsocket) -> Self:
        return cls(
            socket_uid=proto.socket_uid,
            message_received=proto.message_received,
            message_sent=proto.message_sent,
            started=Timestamp.from_proto(proto.started),
            current_time=Timestamp.from_proto(proto.current_time),
            last_message_received=Timestamp.from_proto(proto.last_message_received),
            last_message_sent=Timestamp.from_proto(proto.last_message_sent),
            client_type=proto.client_type,
            remote_ip=proto.remote_ip,
            protocol=proto.protocol,
        )

    @property
    def age(self) -> timedelta:
        return timedelta(
            microseconds=(self.current_time - self.started) // NANOS_PER_MICROS
        )


class ServerInfo(BaseModel):
    socket_uid: str
    age_millis: int
    live_websockets: list[LiveWebsocket] = Field(default_factory=list)

    @classmethod
    def from_proto(cls, proto: responses_pb2.ServerInfo) -> Self:
        return cls(
            socket_uid=proto.socket_uid,
            age_millis=proto.age_millis,
            live_websockets=[
                LiveWebsocket.from_proto(live_websocket)
                for live_websocket in proto.live_websockets
            ],
        )

    def __str__(self) -> str:
        delta = timedelta(seconds=int(0.001 * self.age_millis))
        return (
            f"socket_uid='{self.socket_uid}' "
            f"age={delta} "
            f"open_sockets={len(self.live_websockets)}"
        )


class Ticker(BaseModel):
    symbol: str
    product_symbol: str
    timestamp: AnnotatedTimestamp
    mid: Decimal

    @classmethod
    def from_proto(cls, proto: responses_pb2.Ticker) -> Self:
        return cls(
            symbol=proto.symbol,
            product_symbol=proto.product_symbol,
            timestamp=Timestamp.from_proto(proto.timestamp),
            mid=Decimal(proto.mid.value),
        )


class OrderBookTop(BaseModel):
    buy: PriceAmount
    sell: PriceAmount
    symbol: str
    product_symbol: str
    timestamp: AnnotatedTimestamp

    @classmethod
    def from_proto(cls, proto: responses_pb2.OrderBookTop) -> Self:
        return cls(
            buy=PriceAmount.from_proto(proto.buy),
            sell=PriceAmount.from_proto(proto.sell),
            symbol=proto.symbol,
            product_symbol=proto.product_symbol,
            timestamp=Timestamp.from_proto(proto.timestamp),
        )


class OrderBookTops(BaseModel):
    order_book_tops: list[OrderBookTop]

    @classmethod
    def from_proto(cls, proto: responses_pb2.OrderBookTops) -> Self:
        return cls(
            order_book_tops=[
                OrderBookTop.from_proto(order_book_top)
                for order_book_top in proto.order_book_tops
            ]
        )


class Tickers(BaseModel):
    tickers: list[Ticker]

    @classmethod
    def from_proto(cls, proto: responses_pb2.Tickers) -> Self:
        return cls(
            tickers=[Ticker.from_proto(ticker) for ticker in proto.tickers],
        )


class OtcSubscription(BaseModel):
    channel: Channel
    status: SubscriptionStatus
    message: str

    @classmethod
    def from_proto(cls, proto: responses_pb2.Subscription) -> Self:
        return cls(
            channel=Channel.from_proto(proto.channel),
            status=SubscriptionStatus.from_proto(proto.status),
            message=proto.message,
        )


class OtcError(BaseModel):
    code: OtcErrorCode
    message: str

    @classmethod
    def from_proto(cls, proto: responses_pb2.OtcError) -> Self:
        return cls(
            code=OtcErrorCode.from_proto(proto.code),
            message=proto.message,
        )


class OtcQuote(BaseModel):
    """An Quote from an RFQ subscription"""

    symbol: TradableSymbol
    exchange: Exchange
    timestamp: AnnotatedTimestamp
    product_symbol: str
    buy: PriceAmount
    sell: PriceAmount

    @classmethod
    def from_proto(cls, proto: responses_pb2.OtcQuote) -> Self:
        return cls(
            symbol=TradableSymbol.from_proto(proto.symbol),
            exchange=Exchange.from_proto(proto.exchange),
            timestamp=Timestamp.from_proto(proto.timestamp),
            product_symbol=proto.product_symbol,
            buy=PriceAmount.from_proto(proto.buy),
            sell=PriceAmount.from_proto(proto.sell),
        )

    def as_string(self) -> str:
        return (
            f"{self.symbol.as_string()} "
            f"buy: {self.buy.as_string()}, "
            f"sell: {self.sell.as_string()}"
        )

    @field_validator("symbol", mode="before")
    @classmethod
    def validate_symbol(cls, value: str) -> TradableSymbol:
        """Custom validator to create TradableSymbol from a string."""
        if isinstance(value, str):
            return TradableSymbol.from_string(value)
        return value


class OtcOrder(BaseModel):
    id: str
    client_order_id: str
    account_id: str
    symbol: TradableSymbol
    product_symbol: str
    amount: Decimal
    side: Side
    price: Decimal

    @classmethod
    def from_proto(cls, proto: responses_pb2.Order) -> Self:
        return cls(
            id=proto.id,
            client_order_id=proto.client_order_id,
            account_id=proto.account_id,
            symbol=TradableSymbol.from_proto(proto.symbol),
            product_symbol=proto.product_symbol,
            amount=Decimal(proto.amount.value),
            side=Side.from_proto(proto.side),
            price=Decimal(proto.price.value),
        )


class OtcResponse(BaseModel):
    id: str
    timestamp: AnnotatedTimestamp
    data: Auth | OtcError | OtcSubscription | OtcOrder

    def auth(self) -> Auth | None:
        if isinstance(self.data, Auth):
            return self.data
        return None

    def error(self) -> OtcError | None:
        if isinstance(self.data, OtcError):
            return self.data
        return None

    def subscription(self) -> OtcSubscription | None:
        if isinstance(self.data, OtcSubscription):
            return self.data
        return None

    def order(self) -> OtcOrder | None:
        if isinstance(self.data, OtcOrder):
            return self.data
        return None

    def log(self) -> None:
        name = self.data.__class__.__name__
        if self.error():
            logger.error("%s - %s - %s", self.timestamp, name, self.data)
        else:
            logger.info("%s - %s - %s", self.timestamp, name, self.data)

    @classmethod
    def from_proto(cls, proto: responses_pb2.OtcResponse) -> Self:
        return cls(
            id=proto.id,
            timestamp=Timestamp.from_proto(proto.timestamp),
            data=cls.get_data_from_proto(proto),
        )

    @classmethod
    def get_data_from_proto(
        cls, proto: responses_pb2.OtcResponse
    ) -> Auth | OtcError | OtcSubscription | OtcOrder:
        match proto.WhichOneof("response"):  # type: ignore[arg-type]
            case "auth":
                return Auth.from_proto(proto.auth)
            case "error":
                return OtcError.from_proto(proto.error)
            case "subscription":
                return OtcSubscription.from_proto(proto.subscription)
            case "order":
                return OtcOrder.from_proto(proto.order)
            case _:
                raise ValueError(f"Unknown response type {proto}")

    @classmethod
    def from_json(cls, payload: dict) -> Self | None:
        id_ = payload.get("id")
        if id_ is None:
            return None
        method = payload["method"]
        timestamp = payload["timestamp"]
        match method:
            case "auth":
                return cls(
                    id=id_, timestamp=timestamp, data=Auth(message=payload["message"])
                )
            case "otcerror":
                return cls(
                    id=id_,
                    timestamp=timestamp,
                    data=OtcError(
                        message=payload["message"],
                        code=OtcErrorCode[payload["code"].upper()],
                    ),
                )
            case "subscribe" | "unsubscribe":
                return cls(
                    id=id_,
                    timestamp=timestamp,
                    data=OtcSubscription(
                        channel=payload["channel"],
                        message=payload["message"],
                        status=payload["status"],
                    ),
                )
            case "order":
                return cls(
                    id=id_,
                    timestamp=timestamp,
                    data=OtcOrder(
                        id=payload["id"],
                        client_order_id=payload["client_order_id"],
                        account_id=payload["account_id"],
                        symbol=TradableSymbol.from_string(payload["symbol"]),
                        product_symbol=payload["product_symbol"],
                        amount=Decimal(payload["amount"]),
                        side=Side.from_proto(payload["side"]),
                        price=Decimal(payload["price"]),
                    ),
                )
            case _:
                raise ValueError(f"Unknown method: {method}")


class OtcChannelMessage(BaseModel):
    """A message in a subscribed channel"""

    channel: Channel
    timestamp: AnnotatedTimestamp
    data: ServerInfo | Tickers | OtcQuote | OrderBookTops | OtcOrder

    def server_info(self) -> ServerInfo | None:
        if isinstance(self.data, ServerInfo):
            return self.data
        return None

    def tickers(self) -> Tickers | None:
        if isinstance(self.data, Tickers):
            return self.data
        return None

    def order_book_tops(self) -> OrderBookTops | None:
        if isinstance(self.data, OrderBookTops):
            return self.data
        return None

    def otc_quote(self) -> OtcQuote | None:
        if isinstance(self.data, OtcQuote):
            return self.data
        return None

    def order(self) -> OtcOrder | None:
        if isinstance(self.data, OtcOrder):
            return self.data
        return None

    def log(self) -> None:
        name = self.data.__class__.__name__
        if tickers := self.tickers():
            for ticker in tickers.tickers:
                logger.info("%s - %s - %s", self.timestamp, name, ticker)
        elif order_book_tops := self.order_book_tops():
            for obt in order_book_tops.order_book_tops:
                logger.info("%s - %s - %s", self.timestamp, name, obt)
        else:
            logger.info("%s - %s - %s", self.timestamp, name, self.data)

    @classmethod
    def from_proto(cls, proto: responses_pb2.ChannelMessage) -> Self:
        return cls(
            channel=Channel.from_proto(proto.channel),
            timestamp=Timestamp.from_proto(proto.timestamp),
            data=cls.get_data_from_proto(proto),
        )

    @classmethod
    def get_data_from_proto(
        cls, proto: responses_pb2.ChannelMessage
    ) -> ServerInfo | Tickers | OtcQuote | OrderBookTops | OtcOrder:
        match proto.WhichOneof("message"):  # type: ignore[arg-type]
            case "server_info":
                return ServerInfo.from_proto(proto.server_info)
            case "tickers":
                return Tickers.from_proto(proto.tickers)
            case "otc_quote":
                return OtcQuote.from_proto(proto.otc_quote)
            case "order_book_tops":
                return OrderBookTops.from_proto(proto.order_book_tops)
            case "order":
                return OtcOrder.from_proto(proto.order)
            case _:
                raise ValueError(f"Unknown channel message type {proto}")

    @classmethod
    def from_json(cls, data: dict) -> Self | None:
        channel = getattr(Channel, (data.get("channel") or "").upper(), None)
        if channel is None:
            return None
        timestamp = data["timestamp"]
        message = data["message"]
        match channel:
            case Channel.SERVER_INFO:
                return cls(
                    channel=channel,
                    timestamp=timestamp,
                    data=ServerInfo(**message),
                )
            case Channel.TICKERS:
                return cls(
                    channel=channel,
                    timestamp=timestamp,
                    data=Tickers(tickers=message),
                )
            case Channel.RFQ:
                return cls(
                    channel=channel,
                    timestamp=timestamp,
                    data=OtcQuote(**message),
                )
            case Channel.ORDERS:
                return cls(
                    channel=channel,
                    timestamp=timestamp,
                    data=OtcOrder(**message),
                )
            case Channel.ORDER_BOOK_TOP:
                return cls(
                    channel=channel,
                    timestamp=timestamp,
                    data=OrderBookTops(order_book_tops=message),
                )
            case _:
                raise ValueError(f"Unknown channel: {channel}")


def otc_response_from_proto_bytes(
    proto_bytes: bytes,
) -> OtcResponse | OtcChannelMessage:
    """Convert a protobuf response to an OtcResponse object."""
    msg = responses_pb2.OtcResponseMessage.FromString(proto_bytes)
    match msg.WhichOneof("data"):
        case "otc_response":
            return OtcResponse.from_proto(msg.otc_response)
        case "channel_message":
            return OtcChannelMessage.from_proto(msg.channel_message)
        case _:
            raise ValueError("Unknown response type")



from __future__ import annotations

from decimal import Decimal
from typing import Self

from pydantic import BaseModel

from .v2 import common_pb2, responses_pb2


class InvalidInputError(ValueError):
    pass


class Spread(BaseModel):
    front: str
    back: str

    def to_string(self) -> str:
        return f"{self.front}-{self.back}"


class Butterfly(BaseModel):
    front: str
    middle: str
    back: str

    def to_string(self) -> str:
        return f"{self.front}-{self.middle}-{self.back}"


class PriceAmount(BaseModel):
    price: Decimal
    amount: Decimal

    @classmethod
    def from_proto(
        cls, proto: responses_pb2.PriceAmount | responses_pb2.OtcQuoteSide
    ) -> Self:
        return cls(
            price=Decimal(proto.price.value),
            amount=Decimal(proto.amount.value),
        )

    def as_string(self) -> str:
        return f"{self.amount}@{self.price}"


class TradableSymbol(BaseModel):
    symbol: str | Spread | Butterfly

    @classmethod
    def from_string(cls, symbol: str) -> Self:
        if "-" in symbol:
            parts = symbol.split("-")
            if len(parts) == 2:
                return cls(symbol=Spread(front=parts[0], back=parts[1]))
            if len(parts) == 3:
                return cls(
                    symbol=Butterfly(front=parts[0], middle=parts[1], back=parts[2])
                )
        return cls(symbol=symbol)

    @classmethod
    def from_proto(cls, proto: common_pb2.TradableSymbol) -> Self:
        match proto.WhichOneof("symbol"):
            case "flat":
                return cls(symbol=proto.flat)
            case "spread":
                return cls(
                    symbol=Spread(front=proto.spread.front, back=proto.spread.back)
                )
            case "butterfly":
                return cls(
                    symbol=Butterfly(
                        front=proto.butterfly.front,
                        middle=proto.butterfly.middle,
                        back=proto.butterfly.back,
                    )
                )
            case _:
                raise ValueError(f"Unknown symbol type: {proto}")

    def to_proto(self) -> common_pb2.TradableSymbol:
        if isinstance(self.symbol, str):
            return common_pb2.TradableSymbol(flat=self.symbol)
        elif isinstance(self.symbol, Spread):
            return common_pb2.TradableSymbol(
                spread=common_pb2.Spread(front=self.symbol.front, back=self.symbol.back)
            )
        elif isinstance(self.symbol, Butterfly):
            return common_pb2.TradableSymbol(
                butterfly=common_pb2.Butterfly(
                    front=self.symbol.front,
                    middle=self.symbol.middle,
                    back=self.symbol.back,
                )
            )
        raise ValueError(f"Unknown symbol type: {self.symbol}")

    def as_string(self) -> str:
        if isinstance(self.symbol, str):
            return self.symbol
        return self.symbol.to_string()



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import google.protobuf.timestamp_pb2
import onyx_otc.v2.common_pb2
import onyx_otc.v2.types_pb2
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class NewOrderRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CLIENT_ORDER_ID_FIELD_NUMBER: builtins.int
    ACCOUNT_ID_FIELD_NUMBER: builtins.int
    SYMBOL_FIELD_NUMBER: builtins.int
    QUANTITY_FIELD_NUMBER: builtins.int
    SIDE_FIELD_NUMBER: builtins.int
    ORDER_TYPE_FIELD_NUMBER: builtins.int
    PRICE_FIELD_NUMBER: builtins.int
    EXCHANGE_FIELD_NUMBER: builtins.int
    client_order_id: builtins.str
    account_id: builtins.str
    side: onyx_otc.v2.types_pb2.Side.ValueType
    order_type: onyx_otc.v2.types_pb2.OrderType.ValueType
    exchange: onyx_otc.v2.types_pb2.Exchange.ValueType
    @property
    def symbol(self) -> onyx_otc.v2.common_pb2.TradableSymbol: ...
    @property
    def quantity(self) -> onyx_otc.v2.common_pb2.Decimal: ...
    @property
    def price(self) -> onyx_otc.v2.common_pb2.Decimal: ...
    def __init__(
        self,
        *,
        client_order_id: builtins.str | None = ...,
        account_id: builtins.str = ...,
        symbol: onyx_otc.v2.common_pb2.TradableSymbol | None = ...,
        quantity: onyx_otc.v2.common_pb2.Decimal | None = ...,
        side: onyx_otc.v2.types_pb2.Side.ValueType = ...,
        order_type: onyx_otc.v2.types_pb2.OrderType.ValueType = ...,
        price: onyx_otc.v2.common_pb2.Decimal | None = ...,
        exchange: onyx_otc.v2.types_pb2.Exchange.ValueType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["_client_order_id", b"_client_order_id", "_exchange", b"_exchange", "client_order_id", b"client_order_id", "exchange", b"exchange", "price", b"price", "quantity", b"quantity", "symbol", b"symbol"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["_client_order_id", b"_client_order_id", "_exchange", b"_exchange", "account_id", b"account_id", "client_order_id", b"client_order_id", "exchange", b"exchange", "order_type", b"order_type", "price", b"price", "quantity", b"quantity", "side", b"side", "symbol", b"symbol"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_client_order_id", b"_client_order_id"]) -> typing.Literal["client_order_id"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_exchange", b"_exchange"]) -> typing.Literal["exchange"] | None: ...

global___NewOrderRequest = NewOrderRequest

@typing.final
class TickersChannel(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PRODUCTS_FIELD_NUMBER: builtins.int
    @property
    def products(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        products: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["products", b"products"]) -> None: ...

global___TickersChannel = TickersChannel

@typing.final
class RfqChannel(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SYMBOL_FIELD_NUMBER: builtins.int
    SIZE_FIELD_NUMBER: builtins.int
    EXCHANGE_FIELD_NUMBER: builtins.int
    exchange: onyx_otc.v2.types_pb2.Exchange.ValueType
    @property
    def symbol(self) -> onyx_otc.v2.common_pb2.TradableSymbol: ...
    @property
    def size(self) -> onyx_otc.v2.common_pb2.Decimal: ...
    def __init__(
        self,
        *,
        symbol: onyx_otc.v2.common_pb2.TradableSymbol | None = ...,
        size: onyx_otc.v2.common_pb2.Decimal | None = ...,
        exchange: onyx_otc.v2.types_pb2.Exchange.ValueType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["_exchange", b"_exchange", "exchange", b"exchange", "size", b"size", "symbol", b"symbol"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["_exchange", b"_exchange", "exchange", b"exchange", "size", b"size", "symbol", b"symbol"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["_exchange", b"_exchange"]) -> typing.Literal["exchange"] | None: ...

global___RfqChannel = RfqChannel

@typing.final
class OrderBookTopChannel(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PRODUCTS_FIELD_NUMBER: builtins.int
    @property
    def products(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        products: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["products", b"products"]) -> None: ...

global___OrderBookTopChannel = OrderBookTopChannel

@typing.final
class ServerInfoChannel(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___ServerInfoChannel = ServerInfoChannel

@typing.final
class OrdersChannel(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___OrdersChannel = OrdersChannel

@typing.final
class Auth(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TOKEN_FIELD_NUMBER: builtins.int
    token: builtins.str
    def __init__(
        self,
        *,
        token: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["token", b"token"]) -> None: ...

global___Auth = Auth

@typing.final
class Subscribe(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SERVER_INFO_FIELD_NUMBER: builtins.int
    ORDERS_FIELD_NUMBER: builtins.int
    TICKERS_FIELD_NUMBER: builtins.int
    RFQ_CHANNEL_FIELD_NUMBER: builtins.int
    ORDER_BOOK_TOP_FIELD_NUMBER: builtins.int
    @property
    def server_info(self) -> global___ServerInfoChannel: ...
    @property
    def orders(self) -> global___OrdersChannel: ...
    @property
    def tickers(self) -> global___TickersChannel: ...
    @property
    def rfq_channel(self) -> global___RfqChannel: ...
    @property
    def order_book_top(self) -> global___OrderBookTopChannel: ...
    def __init__(
        self,
        *,
        server_info: global___ServerInfoChannel | None = ...,
        orders: global___OrdersChannel | None = ...,
        tickers: global___TickersChannel | None = ...,
        rfq_channel: global___RfqChannel | None = ...,
        order_book_top: global___OrderBookTopChannel | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["channel", b"channel", "order_book_top", b"order_book_top", "orders", b"orders", "rfq_channel", b"rfq_channel", "server_info", b"server_info", "tickers", b"tickers"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["channel", b"channel", "order_book_top", b"order_book_top", "orders", b"orders", "rfq_channel", b"rfq_channel", "server_info", b"server_info", "tickers", b"tickers"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["channel", b"channel"]) -> typing.Literal["server_info", "orders", "tickers", "rfq_channel", "order_book_top"] | None: ...

global___Subscribe = Subscribe

@typing.final
class Unsubscribe(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SERVER_INFO_FIELD_NUMBER: builtins.int
    ORDERS_FIELD_NUMBER: builtins.int
    TICKERS_FIELD_NUMBER: builtins.int
    RFQ_CHANNEL_FIELD_NUMBER: builtins.int
    ORDER_BOOK_TOP_FIELD_NUMBER: builtins.int
    @property
    def server_info(self) -> global___ServerInfoChannel: ...
    @property
    def orders(self) -> global___OrdersChannel: ...
    @property
    def tickers(self) -> global___TickersChannel: ...
    @property
    def rfq_channel(self) -> global___RfqChannel: ...
    @property
    def order_book_top(self) -> global___OrderBookTopChannel: ...
    def __init__(
        self,
        *,
        server_info: global___ServerInfoChannel | None = ...,
        orders: global___OrdersChannel | None = ...,
        tickers: global___TickersChannel | None = ...,
        rfq_channel: global___RfqChannel | None = ...,
        order_book_top: global___OrderBookTopChannel | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["channel", b"channel", "order_book_top", b"order_book_top", "orders", b"orders", "rfq_channel", b"rfq_channel", "server_info", b"server_info", "tickers", b"tickers"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["channel", b"channel", "order_book_top", b"order_book_top", "orders", b"orders", "rfq_channel", b"rfq_channel", "server_info", b"server_info", "tickers", b"tickers"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["channel", b"channel"]) -> typing.Literal["server_info", "orders", "tickers", "rfq_channel", "order_book_top"] | None: ...

global___Unsubscribe = Unsubscribe

@typing.final
class OtcRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    METHOD_FIELD_NUMBER: builtins.int
    AUTH_FIELD_NUMBER: builtins.int
    SUBSCRIBE_FIELD_NUMBER: builtins.int
    UNSUBSCRIBE_FIELD_NUMBER: builtins.int
    ORDER_FIELD_NUMBER: builtins.int
    id: builtins.str
    method: onyx_otc.v2.types_pb2.Method.ValueType
    @property
    def timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def auth(self) -> global___Auth: ...
    @property
    def subscribe(self) -> global___Subscribe: ...
    @property
    def unsubscribe(self) -> global___Unsubscribe: ...
    @property
    def order(self) -> global___NewOrderRequest: ...
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        method: onyx_otc.v2.types_pb2.Method.ValueType = ...,
        auth: global___Auth | None = ...,
        subscribe: global___Subscribe | None = ...,
        unsubscribe: global___Unsubscribe | None = ...,
        order: global___NewOrderRequest | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["auth", b"auth", "order", b"order", "request", b"request", "subscribe", b"subscribe", "timestamp", b"timestamp", "unsubscribe", b"unsubscribe"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["auth", b"auth", "id", b"id", "method", b"method", "order", b"order", "request", b"request", "subscribe", b"subscribe", "timestamp", b"timestamp", "unsubscribe", b"unsubscribe"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["request", b"request"]) -> typing.Literal["auth", "subscribe", "unsubscribe", "order"] | None: ...

global___OtcRequest = OtcRequest



from datetime import datetime, timezone
from decimal import Decimal
from typing import Any

from .v2 import common_pb2


def to_proto_decimal(value: Decimal) -> common_pb2.Decimal:
    return common_pb2.Decimal(value=str(value))


def isoformat(dt: datetime, **kwargs: Any) -> str:
    if dt.tzinfo is timezone.utc:
        return dt.replace(tzinfo=None).isoformat(**kwargs) + "Z"
    else:
        return dt.isoformat(**kwargs)



from __future__ import annotations

import asyncio
import json
import logging
import os
from dataclasses import dataclass, field
from typing import Any, Callable, Self, TypeAlias

from aiohttp import ClientSession, ClientWebSocketResponse, WSMsgType

from .requests import (
    AuthRequest,
    OrderBookTopChannel,
    OrdersChannel,
    OtcOrderRequest,
    OtcRequest,
    RfqChannel,
    ServerInfoChannel,
    SubscribeRequest,
    TickersChannel,
    UnsubscribeRequest,
)
from .responses import OtcChannelMessage, OtcResponse, otc_response_from_proto_bytes
from .timestamp import Timestamp

logger = logging.getLogger(__name__)

ResponseHandler: TypeAlias = Callable[["OnyxWebsocketClientV2", OtcResponse], None]
EventHandler: TypeAlias = Callable[["OnyxWebsocketClientV2", OtcChannelMessage], None]
ExitHandler: TypeAlias = Callable[["OnyxWebsocketClientV2"], None]


# Default handlers
def on_response(cli: OnyxWebsocketClientV2, response: OtcResponse) -> None:
    logger.info("Received response: %s", response)


def on_event(cli: OnyxWebsocketClientV2, message: OtcChannelMessage) -> None:
    logger.info("Received event: %s", message)


def on_exit(cli: OnyxWebsocketClientV2) -> None:
    logger.info("Connection closed")


@dataclass
class OnyxWebsocketClientV2:
    """
    WebSocket client for the Onyx OTC API v2.

    This clients connects to either the JSON or the binary (protobuf) endpoint

    Attributes:
        ws_url: WebSocket endpoint URL
        api_token: API authentication token
        on_response: Callback for handling responses
        on_event: Callback for handling channel events
        on_exit: Callback for handling connection closure
    """

    ws_url: str
    api_token: str = field(default_factory=lambda: os.environ.get("ONYX_API_TOKEN", ""))
    on_response: ResponseHandler = field(default=on_response)
    on_event: EventHandler = field(default=on_event)
    on_exit: ExitHandler = field(default=on_exit)
    min_reconnect_delay: float = field(default=1.0, init=False)
    max_reconnect_delay: float = field(default=60.0, init=False)

    _queue: asyncio.Queue[bytes | str] = field(
        default_factory=asyncio.Queue, repr=False
    )
    _ws: ClientWebSocketResponse | None = field(default=None, init=False)
    _write_task: asyncio.Task | None = field(default=None, init=False)
    _is_running: bool = field(default=False, init=False)
    _reconnect_delay: float = 0.0
    _id_counter: int = 0

    @classmethod
    def create(
        cls,
        *,
        binary: bool = True,
        ws_url: str | None = None,
        **kwargs: Any,
    ) -> Self:
        """Create a client instance."""
        ws_url_ = (
            ws_url
            or os.environ.get("ONYX_WS_V2_URL")
            or "wss://ws.onyxhub.co/stream/v2/binary"
        )
        if ws_url_.endswith("/binary"):
            if not binary:
                ws_url_ = ws_url_.replace("/binary", "")
        elif binary:
            ws_url_ = f"{ws_url_}/binary"
        params = {key: value for key, value in kwargs.items() if value is not None}
        return cls(ws_url=ws_url_, **params)

    @property
    def is_binary(self) -> bool:
        """Check if the client is using binary protocol."""
        return self.ws_url.endswith("/binary")

    @property
    def is_running(self) -> bool:
        """Check if the client is currently running."""
        return self._is_running

    def authenticate(self) -> None:
        """Authenticate the client."""
        if self.api_token:
            self.send(self.request(AuthRequest(token=self.api_token)))
        else:
            logger.warning("No API token provided, authentication skipped")

    def subscribe_server_info(self) -> None:
        """Subscribe to server info channel."""
        self.send(self.request(SubscribeRequest(data=ServerInfoChannel())))

    def unsubscribe_server_info(self) -> None:
        """Unsubscribe from server info channel."""
        self.send(self.request(UnsubscribeRequest(data=ServerInfoChannel())))

    def subscribe_tickers(self, products: list[str]) -> None:
        """Subscribe to ticker updates for specific products."""
        self.send(
            self.request(SubscribeRequest(data=TickersChannel(products=products)))
        )

    def unsubscribe_tickers(self, products: list[str]) -> None:
        """Unsubscribe from ticker updates for specific products."""
        self.send(
            self.request(UnsubscribeRequest(data=TickersChannel(products=products)))
        )

    def subscribe_obt(self, products: list[str]) -> None:
        """Subscribe to order-book-top updates for specific products."""
        self.send(
            self.request(SubscribeRequest(data=OrderBookTopChannel(products=products)))
        )

    def unsubscribe_obt(self, products: list[str]) -> None:
        """Unsubscribe from order-book-top updates for specific products."""
        self.send(
            self.request(
                UnsubscribeRequest(data=OrderBookTopChannel(products=products))
            )
        )

    def subscribe_orders(self) -> None:
        """Subscribe to order updates."""
        self.send(self.request(SubscribeRequest(data=OrdersChannel())))

    def unsubscribe_orders(self) -> None:
        """Unsubscribe from order updates."""
        self.send(self.request(UnsubscribeRequest(data=OrdersChannel())))

    def subscribe_rfq(self, rfq: RfqChannel) -> None:
        """Subscribe to RFQ updates."""
        self.send(self.request(SubscribeRequest(data=rfq)))

    def unsubscribe_rfq(self, rfq: RfqChannel) -> None:
        """Unsubscribe from RFQ updates."""
        self.send(self.request(UnsubscribeRequest(data=rfq)))

    async def handle_binary_message(self, data: bytes) -> None:
        """Handle incoming binary messages."""
        response = otc_response_from_proto_bytes(data)
        if isinstance(response, OtcResponse):
            self.on_response(self, response)
        elif isinstance(response, OtcChannelMessage):
            self.on_event(self, response)

    async def handle_text_message(self, data: str) -> None:
        """Handle incoming text messages."""
        payload = json.loads(data)
        if response := OtcResponse.from_json(payload):
            self.on_response(self, response)
        elif message := OtcChannelMessage.from_json(payload):
            self.on_event(self, message)
        else:
            logger.warning("Unknown JSON message type received")

    def send(self, msg: OtcRequest) -> None:
        """Queue a message for sending."""
        if not self._is_running:
            logger.warning("Client not running, message dropped: %s", msg)
            return
        if self.is_binary:
            self._queue.put_nowait(msg.to_proto().SerializeToString())
        else:
            self._queue.put_nowait(json.dumps(msg.to_json_dict()))

    def new_id(self) -> str:
        """Generate a new unique ID for requests."""
        self._id_counter += 1
        return f"wscli:{self._id_counter}"

    def request(
        self,
        request: AuthRequest | OtcOrderRequest | SubscribeRequest | UnsubscribeRequest,
    ) -> OtcRequest:
        """Create a new request with the current timestamp."""
        otc_request = OtcRequest(
            id=self.new_id(),
            timestamp=Timestamp.utcnow(),
            request=request,
        )
        logger.debug("Request created: %s", otc_request)
        return otc_request

    async def connect(self) -> None:
        """Connect to the websocket server with automatic reconnection."""
        self._reconnect_delay = self.min_reconnect_delay
        while True:
            try:
                await self._connect_and_run()
            except Exception as e:
                logger.error("Connection error: %s", e, exc_info=True)
                await asyncio.sleep(self._reconnect_delay)
                self._reconnect_delay = min(
                    self._reconnect_delay * 2, self.max_reconnect_delay
                )
            finally:
                self._ws = None
                if self._write_task:
                    self._write_task.cancel()
                    self._write_task = None

    async def _connect_and_run(self) -> None:
        """Establish connection and start message loops."""
        async with ClientSession() as session:
            logger.info("Connecting to %s", self.ws_url)
            async with session.ws_connect(self.ws_url) as ws:
                self._ws = ws
                self._is_running = True
                logger.info("Connected to %s", self.ws_url)
                self._reconnect_delay = self.min_reconnect_delay
                # Start write loop
                self._write_task = asyncio.create_task(self._write_loop())
                # Authenticate
                self.authenticate()
                # Handle incoming messages
                try:
                    async for msg in ws:
                        if msg.type == WSMsgType.BINARY:
                            await self.handle_binary_message(msg.data)
                        elif msg.type == WSMsgType.TEXT:
                            await self.handle_text_message(msg.data)
                        elif msg.type in (
                            WSMsgType.CLOSED,
                            WSMsgType.CLOSE,
                            WSMsgType.ERROR,
                        ):
                            logger.info("WebSocket closed: %s", msg.type)
                            break
                finally:
                    self._is_running = False
                    self.on_exit(self)

    async def _write_loop(self) -> None:
        """Handle outgoing messages."""
        while True:
            try:
                msg = await self._queue.get()
                if self._ws and not self._ws.closed:
                    if isinstance(msg, str):
                        logger.debug("Sending message string: %s", msg)
                        await self._ws.send_str(msg)
                    elif isinstance(msg, bytes):
                        await self._ws.send_bytes(msg)
                else:
                    logger.warning("WebSocket closed, message dropped: %s", msg)
            except asyncio.CancelledError:
                break
            except Exception as e:
                logger.error("Error sending message: %s", e, exc_info=True)

    async def close(self) -> None:
        """Gracefully close the connection."""
        self._is_running = False
        if self._ws:
            await self._ws.close()
        if self._write_task:
            self._write_task.cancel()
            try:
                await self._write_task
            except asyncio.CancelledError:
                pass



from __future__ import annotations

import time
from datetime import date, datetime, timezone
from typing import Any, Self

from google.protobuf import timestamp_pb2
from pydantic import GetCoreSchemaHandler
from pydantic_core import CoreSchema, core_schema

from .utils import isoformat

NANOS_PER_MICROS = 1000
NANOS_PER_MILLIS = 1000 * NANOS_PER_MICROS
NANOS_PER_SECOND = 1000 * NANOS_PER_MILLIS


class Timestamp(int):
    """An UTC timestamp with converstion to and from proto"""

    @property
    def nanos(self) -> int:
        """timestamp as nanoseconds"""
        return self

    @property
    def micros(self) -> int:
        """timestamp as microseconds"""
        return self // NANOS_PER_MICROS

    @property
    def millis(self) -> int:
        """timestamp as milliseconds"""
        return self // NANOS_PER_MILLIS

    @property
    def seconds(self) -> int:
        """timestamp as seconds"""
        return self // NANOS_PER_SECOND

    @property
    def total_seconds(self) -> float:
        return self / NANOS_PER_SECOND

    @property
    def total_millis(self) -> float:
        return self / NANOS_PER_MILLIS

    @classmethod
    def __get_pydantic_core_schema__(
        cls, source_type: Any, handler: GetCoreSchemaHandler
    ) -> CoreSchema:
        return core_schema.no_info_after_validator_function(
            cls,
            handler(int),
            serialization=core_schema.plain_serializer_function_ser_schema(
                lambda value: value.to_datetime(),
                info_arg=False,
            ),
        )

    @classmethod
    def __get_pydantic_json_schema__(
        cls, core_schema: CoreSchema, handler: Any
    ) -> dict:
        json_schema = handler(core_schema)
        json_schema.update(type="string", format="date-time")
        return json_schema

    @classmethod
    def utcnow(cls) -> Self:
        return cls(time.time_ns())

    @classmethod
    def utcnow_plus_millis(cls, millis: int) -> Self:
        return cls(time.time_ns() + millis * NANOS_PER_MILLIS)

    @classmethod
    def from_micros(cls, micros: int) -> Self:
        return cls(micros * NANOS_PER_MICROS)

    @classmethod
    def from_millis(cls, millis: int | float) -> Self:
        return cls(millis * NANOS_PER_MILLIS)

    @classmethod
    def from_seconds(cls, seconds: float, nanos: int = 0) -> Self:
        return cls(seconds * NANOS_PER_SECOND + nanos)

    @classmethod
    def from_any(cls, value: Any) -> Self:
        if isinstance(value, (float, int)):
            return cls(value)
        elif isinstance(value, date):
            return cls.from_datetime(value)
        elif isinstance(value, str):
            return cls.from_iso_string(value)
        raise ValueError(f"Cannot convert {value} to Timestamp")

    @classmethod
    def from_datetime(cls, dte: date | None) -> Self:
        if dte is None:
            return cls(0)
        elif not isinstance(dte, datetime):
            dte = datetime.combine(dte, datetime.min.time())
        return cls(int(dte.timestamp() * NANOS_PER_SECOND))

    @classmethod
    def from_datetime_or_none(cls, dte: date | None) -> Self | None:
        if dte is None:
            return None
        return cls.from_datetime(dte)

    @classmethod
    def from_iso_string(cls, dte: str) -> Self:
        return cls.from_datetime(datetime.fromisoformat(dte))

    @classmethod
    def from_proto(cls, timestamp: timestamp_pb2.Timestamp) -> Self:
        return cls(NANOS_PER_SECOND * timestamp.seconds + timestamp.nanos)

    def to_proto(self) -> timestamp_pb2.Timestamp:
        return timestamp_pb2.Timestamp(
            seconds=self // NANOS_PER_SECOND,
            nanos=self % NANOS_PER_SECOND,
        )

    def to_datetime(self, tzinfo: timezone | None = timezone.utc) -> datetime:
        return datetime.fromtimestamp(self.total_seconds, tz=tzinfo)

    def __repr__(self) -> str:
        return isoformat(self.to_datetime(), timespec="milliseconds")

    def __str__(self) -> str:
        return self.__repr__()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: onyx_otc/v2/requests.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'onyx_otc/v2/requests.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from onyx_otc.v2 import common_pb2 as onyx__otc_dot_v2_dot_common__pb2
from onyx_otc.v2 import types_pb2 as onyx__otc_dot_v2_dot_types__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1aonyx_otc/v2/requests.proto\x12\x0bonyx_otc.v2\x1a\x18onyx_otc/v2/common.proto\x1a\x17onyx_otc/v2/types.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xd9\x02\n\x0fNewOrderRequest\x12\x1c\n\x0f\x63lient_order_id\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x12\n\naccount_id\x18\x02 \x01(\t\x12+\n\x06symbol\x18\x03 \x01(\x0b\x32\x1b.onyx_otc.v2.TradableSymbol\x12&\n\x08quantity\x18\x0b \x01(\x0b\x32\x14.onyx_otc.v2.Decimal\x12\x1f\n\x04side\x18\x0c \x01(\x0e\x32\x11.onyx_otc.v2.Side\x12*\n\norder_type\x18\x0e \x01(\x0e\x32\x16.onyx_otc.v2.OrderType\x12#\n\x05price\x18\x0f \x01(\x0b\x32\x14.onyx_otc.v2.Decimal\x12,\n\x08\x65xchange\x18\x10 \x01(\x0e\x32\x15.onyx_otc.v2.ExchangeH\x01\x88\x01\x01\x42\x12\n\x10_client_order_idB\x0b\n\t_exchange\"\"\n\x0eTickersChannel\x12\x10\n\x08products\x18\x01 \x03(\t\"\x98\x01\n\nRfqChannel\x12+\n\x06symbol\x18\x01 \x01(\x0b\x32\x1b.onyx_otc.v2.TradableSymbol\x12\"\n\x04size\x18\x02 \x01(\x0b\x32\x14.onyx_otc.v2.Decimal\x12,\n\x08\x65xchange\x18\x03 \x01(\x0e\x32\x15.onyx_otc.v2.ExchangeH\x00\x88\x01\x01\x42\x0b\n\t_exchange\"\'\n\x13OrderBookTopChannel\x12\x10\n\x08products\x18\x01 \x03(\t\"\x13\n\x11ServerInfoChannel\"\x0f\n\rOrdersChannel\"\x15\n\x04\x41uth\x12\r\n\x05token\x18\x02 \x01(\t\"\x97\x02\n\tSubscribe\x12\x35\n\x0bserver_info\x18\x01 \x01(\x0b\x32\x1e.onyx_otc.v2.ServerInfoChannelH\x00\x12,\n\x06orders\x18\x02 \x01(\x0b\x32\x1a.onyx_otc.v2.OrdersChannelH\x00\x12.\n\x07tickers\x18\x03 \x01(\x0b\x32\x1b.onyx_otc.v2.TickersChannelH\x00\x12.\n\x0brfq_channel\x18\x04 \x01(\x0b\x32\x17.onyx_otc.v2.RfqChannelH\x00\x12:\n\x0eorder_book_top\x18\x05 \x01(\x0b\x32 .onyx_otc.v2.OrderBookTopChannelH\x00\x42\t\n\x07\x63hannel\"\x99\x02\n\x0bUnsubscribe\x12\x35\n\x0bserver_info\x18\x01 \x01(\x0b\x32\x1e.onyx_otc.v2.ServerInfoChannelH\x00\x12,\n\x06orders\x18\x02 \x01(\x0b\x32\x1a.onyx_otc.v2.OrdersChannelH\x00\x12.\n\x07tickers\x18\x03 \x01(\x0b\x32\x1b.onyx_otc.v2.TickersChannelH\x00\x12.\n\x0brfq_channel\x18\x04 \x01(\x0b\x32\x17.onyx_otc.v2.RfqChannelH\x00\x12:\n\x0eorder_book_top\x18\x05 \x01(\x0b\x32 .onyx_otc.v2.OrderBookTopChannelH\x00\x42\t\n\x07\x63hannel\"\xa7\x02\n\nOtcRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12-\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12#\n\x06method\x18\x03 \x01(\x0e\x32\x13.onyx_otc.v2.Method\x12!\n\x04\x61uth\x18\n \x01(\x0b\x32\x11.onyx_otc.v2.AuthH\x00\x12+\n\tsubscribe\x18\x0b \x01(\x0b\x32\x16.onyx_otc.v2.SubscribeH\x00\x12/\n\x0bunsubscribe\x18\x0c \x01(\x0b\x32\x18.onyx_otc.v2.UnsubscribeH\x00\x12-\n\x05order\x18\r \x01(\x0b\x32\x1c.onyx_otc.v2.NewOrderRequestH\x00\x42\t\n\x07requestB\x0b\xaa\x02\x08Onyx.Otcb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'onyx_otc.v2.requests_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\252\002\010Onyx.Otc'
  _globals['_NEWORDERREQUEST']._serialized_start=128
  _globals['_NEWORDERREQUEST']._serialized_end=473
  _globals['_TICKERSCHANNEL']._serialized_start=475
  _globals['_TICKERSCHANNEL']._serialized_end=509
  _globals['_RFQCHANNEL']._serialized_start=512
  _globals['_RFQCHANNEL']._serialized_end=664
  _globals['_ORDERBOOKTOPCHANNEL']._serialized_start=666
  _globals['_ORDERBOOKTOPCHANNEL']._serialized_end=705
  _globals['_SERVERINFOCHANNEL']._serialized_start=707
  _globals['_SERVERINFOCHANNEL']._serialized_end=726
  _globals['_ORDERSCHANNEL']._serialized_start=728
  _globals['_ORDERSCHANNEL']._serialized_end=743
  _globals['_AUTH']._serialized_start=745
  _globals['_AUTH']._serialized_end=766
  _globals['_SUBSCRIBE']._serialized_start=769
  _globals['_SUBSCRIBE']._serialized_end=1048
  _globals['_UNSUBSCRIBE']._serialized_start=1051
  _globals['_UNSUBSCRIBE']._serialized_end=1332
  _globals['_OTCREQUEST']._serialized_start=1335
  _globals['_OTCREQUEST']._serialized_end=1630
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import google.protobuf.timestamp_pb2
import onyx_otc.v2.common_pb2
import onyx_otc.v2.types_pb2
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class Trader(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    ACCOUNT_NAME_FIELD_NUMBER: builtins.int
    FULL_NAME_FIELD_NUMBER: builtins.int
    EMAIL_FIELD_NUMBER: builtins.int
    id: builtins.str
    account_name: builtins.str
    full_name: builtins.str
    email: builtins.str
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        account_name: builtins.str = ...,
        full_name: builtins.str = ...,
        email: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["account_name", b"account_name", "email", b"email", "full_name", b"full_name", "id", b"id"]) -> None: ...

global___Trader = Trader

@typing.final
class PriceAmount(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PRICE_FIELD_NUMBER: builtins.int
    AMOUNT_FIELD_NUMBER: builtins.int
    @property
    def price(self) -> onyx_otc.v2.common_pb2.Decimal: ...
    @property
    def amount(self) -> onyx_otc.v2.common_pb2.Decimal: ...
    def __init__(
        self,
        *,
        price: onyx_otc.v2.common_pb2.Decimal | None = ...,
        amount: onyx_otc.v2.common_pb2.Decimal | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["amount", b"amount", "price", b"price"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["amount", b"amount", "price", b"price"]) -> None: ...

global___PriceAmount = PriceAmount

@typing.final
class Ticker(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SYMBOL_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    MID_FIELD_NUMBER: builtins.int
    FULL_PRICE_FIELD_NUMBER: builtins.int
    ACCRUED_PRICE_FIELD_NUMBER: builtins.int
    PRODUCT_SYMBOL_FIELD_NUMBER: builtins.int
    symbol: builtins.str
    product_symbol: builtins.str
    @property
    def timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def mid(self) -> onyx_otc.v2.common_pb2.Decimal: ...
    @property
    def full_price(self) -> onyx_otc.v2.common_pb2.Decimal: ...
    @property
    def accrued_price(self) -> onyx_otc.v2.common_pb2.Decimal: ...
    def __init__(
        self,
        *,
        symbol: builtins.str = ...,
        timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        mid: onyx_otc.v2.common_pb2.Decimal | None = ...,
        full_price: onyx_otc.v2.common_pb2.Decimal | None = ...,
        accrued_price: onyx_otc.v2.common_pb2.Decimal | None = ...,
        product_symbol: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["accrued_price", b"accrued_price", "full_price", b"full_price", "mid", b"mid", "timestamp", b"timestamp"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["accrued_price", b"accrued_price", "full_price", b"full_price", "mid", b"mid", "product_symbol", b"product_symbol", "symbol", b"symbol", "timestamp", b"timestamp"]) -> None: ...

global___Ticker = Ticker

@typing.final
class Fee(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CURRENCY_FIELD_NUMBER: builtins.int
    SIGNED_AMOUNT_FIELD_NUMBER: builtins.int
    RATE_FIELD_NUMBER: builtins.int
    currency: builtins.str
    @property
    def signed_amount(self) -> onyx_otc.v2.common_pb2.Decimal: ...
    @property
    def rate(self) -> onyx_otc.v2.common_pb2.Decimal: ...
    def __init__(
        self,
        *,
        currency: builtins.str = ...,
        signed_amount: onyx_otc.v2.common_pb2.Decimal | None = ...,
        rate: onyx_otc.v2.common_pb2.Decimal | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["rate", b"rate", "signed_amount", b"signed_amount"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["currency", b"currency", "rate", b"rate", "signed_amount", b"signed_amount"]) -> None: ...

global___Fee = Fee

@typing.final
class Execution(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    ORDER_ID_FIELD_NUMBER: builtins.int
    CLIENT_ORDER_ID_FIELD_NUMBER: builtins.int
    ACCOUNT_ID_FIELD_NUMBER: builtins.int
    EXCHANGE_FIELD_NUMBER: builtins.int
    SYMBOL_FIELD_NUMBER: builtins.int
    AMOUNT_FIELD_NUMBER: builtins.int
    PRICE_FIELD_NUMBER: builtins.int
    SIDE_FIELD_NUMBER: builtins.int
    CREATED_TIMESTAMP_FIELD_NUMBER: builtins.int
    FEE_FIELD_NUMBER: builtins.int
    USER_FIELD_NUMBER: builtins.int
    id: builtins.str
    order_id: builtins.str
    client_order_id: builtins.str
    account_id: builtins.str
    exchange: onyx_otc.v2.types_pb2.Exchange.ValueType
    symbol: builtins.str
    side: onyx_otc.v2.types_pb2.Side.ValueType
    @property
    def amount(self) -> onyx_otc.v2.common_pb2.Decimal: ...
    @property
    def price(self) -> onyx_otc.v2.common_pb2.Decimal: ...
    @property
    def created_timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def fee(self) -> global___Fee: ...
    @property
    def user(self) -> global___Trader: ...
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        order_id: builtins.str = ...,
        client_order_id: builtins.str = ...,
        account_id: builtins.str = ...,
        exchange: onyx_otc.v2.types_pb2.Exchange.ValueType = ...,
        symbol: builtins.str = ...,
        amount: onyx_otc.v2.common_pb2.Decimal | None = ...,
        price: onyx_otc.v2.common_pb2.Decimal | None = ...,
        side: onyx_otc.v2.types_pb2.Side.ValueType = ...,
        created_timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        fee: global___Fee | None = ...,
        user: global___Trader | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["amount", b"amount", "created_timestamp", b"created_timestamp", "fee", b"fee", "price", b"price", "user", b"user"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["account_id", b"account_id", "amount", b"amount", "client_order_id", b"client_order_id", "created_timestamp", b"created_timestamp", "exchange", b"exchange", "fee", b"fee", "id", b"id", "order_id", b"order_id", "price", b"price", "side", b"side", "symbol", b"symbol", "user", b"user"]) -> None: ...

global___Execution = Execution

@typing.final
class Tickers(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TICKERS_FIELD_NUMBER: builtins.int
    @property
    def tickers(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Ticker]: ...
    def __init__(
        self,
        *,
        tickers: collections.abc.Iterable[global___Ticker] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["tickers", b"tickers"]) -> None: ...

global___Tickers = Tickers

@typing.final
class LiveWebsocket(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SOCKET_UID_FIELD_NUMBER: builtins.int
    MESSAGE_RECEIVED_FIELD_NUMBER: builtins.int
    MESSAGE_SENT_FIELD_NUMBER: builtins.int
    STARTED_FIELD_NUMBER: builtins.int
    CURRENT_TIME_FIELD_NUMBER: builtins.int
    LAST_MESSAGE_RECEIVED_FIELD_NUMBER: builtins.int
    LAST_MESSAGE_SENT_FIELD_NUMBER: builtins.int
    CLIENT_TYPE_FIELD_NUMBER: builtins.int
    REMOTE_IP_FIELD_NUMBER: builtins.int
    PROTOCOL_FIELD_NUMBER: builtins.int
    socket_uid: builtins.str
    message_received: builtins.int
    message_sent: builtins.int
    client_type: builtins.str
    remote_ip: builtins.str
    protocol: builtins.str
    @property
    def started(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def current_time(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def last_message_received(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def last_message_sent(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    def __init__(
        self,
        *,
        socket_uid: builtins.str = ...,
        message_received: builtins.int = ...,
        message_sent: builtins.int = ...,
        started: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        current_time: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        last_message_received: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        last_message_sent: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        client_type: builtins.str = ...,
        remote_ip: builtins.str = ...,
        protocol: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["current_time", b"current_time", "last_message_received", b"last_message_received", "last_message_sent", b"last_message_sent", "started", b"started"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["client_type", b"client_type", "current_time", b"current_time", "last_message_received", b"last_message_received", "last_message_sent", b"last_message_sent", "message_received", b"message_received", "message_sent", b"message_sent", "protocol", b"protocol", "remote_ip", b"remote_ip", "socket_uid", b"socket_uid", "started", b"started"]) -> None: ...

global___LiveWebsocket = LiveWebsocket

@typing.final
class ServerInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SOCKET_UID_FIELD_NUMBER: builtins.int
    AGE_MILLIS_FIELD_NUMBER: builtins.int
    LIVE_WEBSOCKETS_FIELD_NUMBER: builtins.int
    socket_uid: builtins.str
    age_millis: builtins.int
    @property
    def live_websockets(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___LiveWebsocket]: ...
    def __init__(
        self,
        *,
        socket_uid: builtins.str = ...,
        age_millis: builtins.int = ...,
        live_websockets: collections.abc.Iterable[global___LiveWebsocket] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["age_millis", b"age_millis", "live_websockets", b"live_websockets", "socket_uid", b"socket_uid"]) -> None: ...

global___ServerInfo = ServerInfo

@typing.final
class OtcQuoteSide(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PRICE_FIELD_NUMBER: builtins.int
    AMOUNT_FIELD_NUMBER: builtins.int
    @property
    def price(self) -> onyx_otc.v2.common_pb2.Decimal: ...
    @property
    def amount(self) -> onyx_otc.v2.common_pb2.Decimal: ...
    def __init__(
        self,
        *,
        price: onyx_otc.v2.common_pb2.Decimal | None = ...,
        amount: onyx_otc.v2.common_pb2.Decimal | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["amount", b"amount", "price", b"price"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["amount", b"amount", "price", b"price"]) -> None: ...

global___OtcQuoteSide = OtcQuoteSide

@typing.final
class OtcQuote(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SYMBOL_FIELD_NUMBER: builtins.int
    EXCHANGE_FIELD_NUMBER: builtins.int
    BUY_FIELD_NUMBER: builtins.int
    SELL_FIELD_NUMBER: builtins.int
    MID_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    PRODUCT_SYMBOL_FIELD_NUMBER: builtins.int
    exchange: onyx_otc.v2.types_pb2.Exchange.ValueType
    product_symbol: builtins.str
    @property
    def symbol(self) -> onyx_otc.v2.common_pb2.TradableSymbol: ...
    @property
    def buy(self) -> global___OtcQuoteSide: ...
    @property
    def sell(self) -> global___OtcQuoteSide: ...
    @property
    def mid(self) -> onyx_otc.v2.common_pb2.Decimal: ...
    @property
    def timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    def __init__(
        self,
        *,
        symbol: onyx_otc.v2.common_pb2.TradableSymbol | None = ...,
        exchange: onyx_otc.v2.types_pb2.Exchange.ValueType = ...,
        buy: global___OtcQuoteSide | None = ...,
        sell: global___OtcQuoteSide | None = ...,
        mid: onyx_otc.v2.common_pb2.Decimal | None = ...,
        timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        product_symbol: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["buy", b"buy", "mid", b"mid", "sell", b"sell", "symbol", b"symbol", "timestamp", b"timestamp"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["buy", b"buy", "exchange", b"exchange", "mid", b"mid", "product_symbol", b"product_symbol", "sell", b"sell", "symbol", b"symbol", "timestamp", b"timestamp"]) -> None: ...

global___OtcQuote = OtcQuote

@typing.final
class OrderBookTop(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BUY_FIELD_NUMBER: builtins.int
    SELL_FIELD_NUMBER: builtins.int
    EXCHANGE_FIELD_NUMBER: builtins.int
    SYMBOL_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    PRODUCT_SYMBOL_FIELD_NUMBER: builtins.int
    exchange: onyx_otc.v2.types_pb2.Exchange.ValueType
    symbol: builtins.str
    product_symbol: builtins.str
    @property
    def buy(self) -> global___PriceAmount: ...
    @property
    def sell(self) -> global___PriceAmount: ...
    @property
    def timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    def __init__(
        self,
        *,
        buy: global___PriceAmount | None = ...,
        sell: global___PriceAmount | None = ...,
        exchange: onyx_otc.v2.types_pb2.Exchange.ValueType = ...,
        symbol: builtins.str = ...,
        timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        product_symbol: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["buy", b"buy", "sell", b"sell", "timestamp", b"timestamp"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["buy", b"buy", "exchange", b"exchange", "product_symbol", b"product_symbol", "sell", b"sell", "symbol", b"symbol", "timestamp", b"timestamp"]) -> None: ...

global___OrderBookTop = OrderBookTop

@typing.final
class OrderBookTops(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ORDER_BOOK_TOPS_FIELD_NUMBER: builtins.int
    @property
    def order_book_tops(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___OrderBookTop]: ...
    def __init__(
        self,
        *,
        order_book_tops: collections.abc.Iterable[global___OrderBookTop] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["order_book_tops", b"order_book_tops"]) -> None: ...

global___OrderBookTops = OrderBookTops

@typing.final
class Order(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    CLIENT_ORDER_ID_FIELD_NUMBER: builtins.int
    USER_FIELD_NUMBER: builtins.int
    ACCOUNT_ID_FIELD_NUMBER: builtins.int
    EXCHANGE_FIELD_NUMBER: builtins.int
    PRODUCT_SYMBOL_FIELD_NUMBER: builtins.int
    SYMBOL_FIELD_NUMBER: builtins.int
    ORDER_TYPE_FIELD_NUMBER: builtins.int
    ORDER_STATE_FIELD_NUMBER: builtins.int
    AMOUNT_FIELD_NUMBER: builtins.int
    PRICE_FIELD_NUMBER: builtins.int
    SIDE_FIELD_NUMBER: builtins.int
    CREATED_TIMESTAMP_FIELD_NUMBER: builtins.int
    EXECUTED_AVG_PRICE_FIELD_NUMBER: builtins.int
    EXECUTED_AMOUNT_FIELD_NUMBER: builtins.int
    EXECUTIONS_FIELD_NUMBER: builtins.int
    id: builtins.str
    client_order_id: builtins.str
    account_id: builtins.str
    exchange: onyx_otc.v2.types_pb2.Exchange.ValueType
    product_symbol: builtins.str
    order_type: onyx_otc.v2.types_pb2.OrderType.ValueType
    order_state: onyx_otc.v2.types_pb2.OrderState.ValueType
    side: onyx_otc.v2.types_pb2.Side.ValueType
    @property
    def user(self) -> global___Trader: ...
    @property
    def symbol(self) -> onyx_otc.v2.common_pb2.TradableSymbol: ...
    @property
    def amount(self) -> onyx_otc.v2.common_pb2.Decimal: ...
    @property
    def price(self) -> onyx_otc.v2.common_pb2.Decimal: ...
    @property
    def created_timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def executed_avg_price(self) -> onyx_otc.v2.common_pb2.Decimal: ...
    @property
    def executed_amount(self) -> onyx_otc.v2.common_pb2.Decimal: ...
    @property
    def executions(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Execution]: ...
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        client_order_id: builtins.str = ...,
        user: global___Trader | None = ...,
        account_id: builtins.str = ...,
        exchange: onyx_otc.v2.types_pb2.Exchange.ValueType = ...,
        product_symbol: builtins.str = ...,
        symbol: onyx_otc.v2.common_pb2.TradableSymbol | None = ...,
        order_type: onyx_otc.v2.types_pb2.OrderType.ValueType = ...,
        order_state: onyx_otc.v2.types_pb2.OrderState.ValueType = ...,
        amount: onyx_otc.v2.common_pb2.Decimal | None = ...,
        price: onyx_otc.v2.common_pb2.Decimal | None = ...,
        side: onyx_otc.v2.types_pb2.Side.ValueType = ...,
        created_timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        executed_avg_price: onyx_otc.v2.common_pb2.Decimal | None = ...,
        executed_amount: onyx_otc.v2.common_pb2.Decimal | None = ...,
        executions: collections.abc.Iterable[global___Execution] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["_executed_amount", b"_executed_amount", "_executed_avg_price", b"_executed_avg_price", "_price", b"_price", "amount", b"amount", "created_timestamp", b"created_timestamp", "executed_amount", b"executed_amount", "executed_avg_price", b"executed_avg_price", "price", b"price", "symbol", b"symbol", "user", b"user"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["_executed_amount", b"_executed_amount", "_executed_avg_price", b"_executed_avg_price", "_price", b"_price", "account_id", b"account_id", "amount", b"amount", "client_order_id", b"client_order_id", "created_timestamp", b"created_timestamp", "exchange", b"exchange", "executed_amount", b"executed_amount", "executed_avg_price", b"executed_avg_price", "executions", b"executions", "id", b"id", "order_state", b"order_state", "order_type", b"order_type", "price", b"price", "product_symbol", b"product_symbol", "side", b"side", "symbol", b"symbol", "user", b"user"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_executed_amount", b"_executed_amount"]) -> typing.Literal["executed_amount"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_executed_avg_price", b"_executed_avg_price"]) -> typing.Literal["executed_avg_price"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_price", b"_price"]) -> typing.Literal["price"] | None: ...

global___Order = Order

@typing.final
class ChannelMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CHANNEL_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    ORDER_FIELD_NUMBER: builtins.int
    SERVER_INFO_FIELD_NUMBER: builtins.int
    TICKERS_FIELD_NUMBER: builtins.int
    OTC_QUOTE_FIELD_NUMBER: builtins.int
    ORDER_BOOK_TOPS_FIELD_NUMBER: builtins.int
    channel: onyx_otc.v2.types_pb2.Channel.ValueType
    @property
    def timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def order(self) -> global___Order: ...
    @property
    def server_info(self) -> global___ServerInfo: ...
    @property
    def tickers(self) -> global___Tickers: ...
    @property
    def otc_quote(self) -> global___OtcQuote: ...
    @property
    def order_book_tops(self) -> global___OrderBookTops: ...
    def __init__(
        self,
        *,
        channel: onyx_otc.v2.types_pb2.Channel.ValueType = ...,
        timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        order: global___Order | None = ...,
        server_info: global___ServerInfo | None = ...,
        tickers: global___Tickers | None = ...,
        otc_quote: global___OtcQuote | None = ...,
        order_book_tops: global___OrderBookTops | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["message", b"message", "order", b"order", "order_book_tops", b"order_book_tops", "otc_quote", b"otc_quote", "server_info", b"server_info", "tickers", b"tickers", "timestamp", b"timestamp"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["channel", b"channel", "message", b"message", "order", b"order", "order_book_tops", b"order_book_tops", "otc_quote", b"otc_quote", "server_info", b"server_info", "tickers", b"tickers", "timestamp", b"timestamp"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["message", b"message"]) -> typing.Literal["order", "server_info", "tickers", "otc_quote", "order_book_tops"] | None: ...

global___ChannelMessage = ChannelMessage

@typing.final
class Subscription(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CHANNEL_FIELD_NUMBER: builtins.int
    MESSAGE_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    channel: onyx_otc.v2.types_pb2.Channel.ValueType
    message: builtins.str
    status: onyx_otc.v2.types_pb2.SubscriptionStatus.ValueType
    def __init__(
        self,
        *,
        channel: onyx_otc.v2.types_pb2.Channel.ValueType = ...,
        message: builtins.str = ...,
        status: onyx_otc.v2.types_pb2.SubscriptionStatus.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["channel", b"channel", "message", b"message", "status", b"status"]) -> None: ...

global___Subscription = Subscription

@typing.final
class AuthResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MESSAGE_FIELD_NUMBER: builtins.int
    message: builtins.str
    def __init__(
        self,
        *,
        message: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["message", b"message"]) -> None: ...

global___AuthResponse = AuthResponse

@typing.final
class OtcError(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MESSAGE_FIELD_NUMBER: builtins.int
    CODE_FIELD_NUMBER: builtins.int
    message: builtins.str
    code: onyx_otc.v2.types_pb2.OtcErrorCode.ValueType
    def __init__(
        self,
        *,
        message: builtins.str = ...,
        code: onyx_otc.v2.types_pb2.OtcErrorCode.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["code", b"code", "message", b"message"]) -> None: ...

global___OtcError = OtcError

@typing.final
class OtcResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    ORDER_FIELD_NUMBER: builtins.int
    SUBSCRIPTION_FIELD_NUMBER: builtins.int
    AUTH_FIELD_NUMBER: builtins.int
    ERROR_FIELD_NUMBER: builtins.int
    id: builtins.str
    @property
    def timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def order(self) -> global___Order: ...
    @property
    def subscription(self) -> global___Subscription: ...
    @property
    def auth(self) -> global___AuthResponse: ...
    @property
    def error(self) -> global___OtcError: ...
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        order: global___Order | None = ...,
        subscription: global___Subscription | None = ...,
        auth: global___AuthResponse | None = ...,
        error: global___OtcError | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["auth", b"auth", "error", b"error", "order", b"order", "response", b"response", "subscription", b"subscription", "timestamp", b"timestamp"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["auth", b"auth", "error", b"error", "id", b"id", "order", b"order", "response", b"response", "subscription", b"subscription", "timestamp", b"timestamp"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["response", b"response"]) -> typing.Literal["order", "subscription", "auth", "error"] | None: ...

global___OtcResponse = OtcResponse

@typing.final
class OtcResponseMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OTC_RESPONSE_FIELD_NUMBER: builtins.int
    CHANNEL_MESSAGE_FIELD_NUMBER: builtins.int
    @property
    def otc_response(self) -> global___OtcResponse: ...
    @property
    def channel_message(self) -> global___ChannelMessage: ...
    def __init__(
        self,
        *,
        otc_response: global___OtcResponse | None = ...,
        channel_message: global___ChannelMessage | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["channel_message", b"channel_message", "data", b"data", "otc_response", b"otc_response"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["channel_message", b"channel_message", "data", b"data", "otc_response", b"otc_response"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["data", b"data"]) -> typing.Literal["otc_response", "channel_message"] | None: ...

global___OtcResponseMessage = OtcResponseMessage



from __future__ import annotations

import enum
from typing import Self

from .v2 import types_pb2


class ProtoEnum(enum.StrEnum):

    def __repr__(self) -> str:
        return self.value


class Exchange(ProtoEnum):
    UNSPECIFIED = enum.auto()
    ICE = enum.auto()
    CME = enum.auto()

    @classmethod
    def from_proto(cls, proto: types_pb2.Exchange.ValueType) -> Self:
        return cls[types_pb2.Exchange.Name(proto)[9:]]

    def to_proto(self) -> types_pb2.Exchange.ValueType:
        return getattr(types_pb2.Exchange, f"EXCHANGE_{self.name}")

    def __str__(self) -> str:
        return self.value


class Method(ProtoEnum):
    UNSPECIFIED = enum.auto()
    AUTH = enum.auto()
    ORDER = enum.auto()
    SUBSCRIBE = enum.auto()
    UNSUBSCRIBE = enum.auto()

    @classmethod
    def from_proto(cls, proto: types_pb2.Method.ValueType) -> Self:
        return cls[types_pb2.Method.Name(proto)[7:]]

    def to_proto(self) -> types_pb2.Method.ValueType:
        return getattr(types_pb2.Method, f"METHOD_{self.name}")


class Channel(ProtoEnum):
    UNSPECIFIED = enum.auto()
    SERVER_INFO = enum.auto()
    TICKERS = enum.auto()
    ORDERS = enum.auto()
    ORDER_BOOK_TOP = enum.auto()
    RFQ = enum.auto()

    @classmethod
    def from_proto(cls, proto: types_pb2.Channel.ValueType) -> Self:
        return cls[types_pb2.Channel.Name(proto)[8:]]

    def to_proto(self) -> types_pb2.Channel.ValueType:
        return getattr(types_pb2.Channel, f"CHANNEL_{self.name}")

    @property
    def subscribe_proto_key(self) -> str:
        # TODO: this needs fixing on a new version
        if self is Channel.RFQ:
            return "rfq_channel"
        return self.value


class OrderType(ProtoEnum):
    UNSPECIFIED = enum.auto()
    FILL_OR_KILL = enum.auto()

    @classmethod
    def from_proto(cls, proto: types_pb2.OrderType.ValueType) -> Self:
        return cls[types_pb2.OrderType.Name(proto)[11:]]

    def to_proto(self) -> types_pb2.OrderType.ValueType:
        return getattr(types_pb2.OrderType, f"ORDER_TYPE_{self.name}")


class Side(ProtoEnum):
    UNSPECIFIED = enum.auto()
    BUY = enum.auto()
    SELL = enum.auto()

    @classmethod
    def from_proto(cls, proto: types_pb2.Side.ValueType) -> Self:
        return cls[types_pb2.Side.Name(proto)[5:]]

    def to_proto(self) -> types_pb2.Side.ValueType:
        return getattr(types_pb2.Side, f"SIDE_{self.name}")


class SubscriptionStatus(ProtoEnum):
    UNSPECIFIED = enum.auto()
    SUBSCRIBED = enum.auto()
    UNSUBSCRIBED = enum.auto()

    @classmethod
    def from_proto(cls, proto: types_pb2.SubscriptionStatus.ValueType) -> Self:
        return cls[types_pb2.SubscriptionStatus.Name(proto)[20:]]

    def to_proto(self) -> types_pb2.SubscriptionStatus.ValueType:
        return getattr(types_pb2.SubscriptionStatus, f"SUBSCRIPTION_STATUS_{self.name}")


class OtcErrorCode(ProtoEnum):
    UNSPECIFIED = enum.auto()
    INVALID_REQUEST = enum.auto()
    NOT_IMPLEMENTED = enum.auto()
    UNAUTHENTICATED = enum.auto()
    TOO_MANY_REQUESTS = enum.auto()
    NOT_SUBSCRIBED = enum.auto()
    FORBIDDEN = enum.auto()
    INTERNAL_SERVER_ERROR = enum.auto()

    @classmethod
    def from_proto(cls, proto: types_pb2.OtcErrorCode.ValueType) -> Self:
        return cls[types_pb2.OtcErrorCode.Name(proto)[15:]]

    def to_proto(self) -> types_pb2.OtcErrorCode.ValueType:
        return getattr(types_pb2.OtcErrorCode, f"OTC_ERROR_CODE_{self.name}")



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _OtcErrorCode:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _OtcErrorCodeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_OtcErrorCode.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    OTC_ERROR_CODE_UNSPECIFIED: _OtcErrorCode.ValueType  # 0
    OTC_ERROR_CODE_INVALID_REQUEST: _OtcErrorCode.ValueType  # 1
    OTC_ERROR_CODE_NOT_IMPLEMENTED: _OtcErrorCode.ValueType  # 2
    OTC_ERROR_CODE_UNAUTHENTICATED: _OtcErrorCode.ValueType  # 3
    OTC_ERROR_CODE_TOO_MANY_REQUESTS: _OtcErrorCode.ValueType  # 4
    OTC_ERROR_CODE_NOT_SUBSCRIBED: _OtcErrorCode.ValueType  # 5
    OTC_ERROR_CODE_FORBIDDEN: _OtcErrorCode.ValueType  # 6
    OTC_ERROR_CODE_INTERNAL_SERVER_ERROR: _OtcErrorCode.ValueType  # 30

class OtcErrorCode(_OtcErrorCode, metaclass=_OtcErrorCodeEnumTypeWrapper): ...

OTC_ERROR_CODE_UNSPECIFIED: OtcErrorCode.ValueType  # 0
OTC_ERROR_CODE_INVALID_REQUEST: OtcErrorCode.ValueType  # 1
OTC_ERROR_CODE_NOT_IMPLEMENTED: OtcErrorCode.ValueType  # 2
OTC_ERROR_CODE_UNAUTHENTICATED: OtcErrorCode.ValueType  # 3
OTC_ERROR_CODE_TOO_MANY_REQUESTS: OtcErrorCode.ValueType  # 4
OTC_ERROR_CODE_NOT_SUBSCRIBED: OtcErrorCode.ValueType  # 5
OTC_ERROR_CODE_FORBIDDEN: OtcErrorCode.ValueType  # 6
OTC_ERROR_CODE_INTERNAL_SERVER_ERROR: OtcErrorCode.ValueType  # 30
global___OtcErrorCode = OtcErrorCode

class _Side:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _SideEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_Side.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    SIDE_UNSPECIFIED: _Side.ValueType  # 0
    SIDE_BUY: _Side.ValueType  # 1
    SIDE_SELL: _Side.ValueType  # 2

class Side(_Side, metaclass=_SideEnumTypeWrapper): ...

SIDE_UNSPECIFIED: Side.ValueType  # 0
SIDE_BUY: Side.ValueType  # 1
SIDE_SELL: Side.ValueType  # 2
global___Side = Side

class _SubscriptionStatus:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _SubscriptionStatusEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_SubscriptionStatus.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    SUBSCRIPTION_STATUS_UNSPECIFIED: _SubscriptionStatus.ValueType  # 0
    SUBSCRIPTION_STATUS_SUBSCRIBED: _SubscriptionStatus.ValueType  # 1
    SUBSCRIPTION_STATUS_UNSUBSCRIBED: _SubscriptionStatus.ValueType  # 2

class SubscriptionStatus(_SubscriptionStatus, metaclass=_SubscriptionStatusEnumTypeWrapper): ...

SUBSCRIPTION_STATUS_UNSPECIFIED: SubscriptionStatus.ValueType  # 0
SUBSCRIPTION_STATUS_SUBSCRIBED: SubscriptionStatus.ValueType  # 1
SUBSCRIPTION_STATUS_UNSUBSCRIBED: SubscriptionStatus.ValueType  # 2
global___SubscriptionStatus = SubscriptionStatus

class _Method:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _MethodEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_Method.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    METHOD_UNSPECIFIED: _Method.ValueType  # 0
    METHOD_AUTH: _Method.ValueType  # 1
    METHOD_ORDER: _Method.ValueType  # 2
    METHOD_SUBSCRIBE: _Method.ValueType  # 3
    METHOD_UNSUBSCRIBE: _Method.ValueType  # 4

class Method(_Method, metaclass=_MethodEnumTypeWrapper): ...

METHOD_UNSPECIFIED: Method.ValueType  # 0
METHOD_AUTH: Method.ValueType  # 1
METHOD_ORDER: Method.ValueType  # 2
METHOD_SUBSCRIBE: Method.ValueType  # 3
METHOD_UNSUBSCRIBE: Method.ValueType  # 4
global___Method = Method

class _Exchange:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _ExchangeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_Exchange.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    EXCHANGE_UNSPECIFIED: _Exchange.ValueType  # 0
    EXCHANGE_ICE: _Exchange.ValueType  # 1
    EXCHANGE_CME: _Exchange.ValueType  # 2

class Exchange(_Exchange, metaclass=_ExchangeEnumTypeWrapper): ...

EXCHANGE_UNSPECIFIED: Exchange.ValueType  # 0
EXCHANGE_ICE: Exchange.ValueType  # 1
EXCHANGE_CME: Exchange.ValueType  # 2
global___Exchange = Exchange

class _OrderType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _OrderTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_OrderType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    ORDER_TYPE_UNSPECIFIED: _OrderType.ValueType  # 0
    ORDER_TYPE_FILL_OR_KILL: _OrderType.ValueType  # 1

class OrderType(_OrderType, metaclass=_OrderTypeEnumTypeWrapper): ...

ORDER_TYPE_UNSPECIFIED: OrderType.ValueType  # 0
ORDER_TYPE_FILL_OR_KILL: OrderType.ValueType  # 1
global___OrderType = OrderType

class _OrderState:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _OrderStateEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_OrderState.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    ORDER_STATE_UNSPECIFIED: _OrderState.ValueType  # 0
    ORDER_STATE_PENDING_CREATED: _OrderState.ValueType  # 1
    ORDER_STATE_CREATED: _OrderState.ValueType  # 2
    ORDER_STATE_NEW: _OrderState.ValueType  # 3
    ORDER_STATE_FILLED: _OrderState.ValueType  # 6
    ORDER_STATE_ERRORED: _OrderState.ValueType  # 10

class OrderState(_OrderState, metaclass=_OrderStateEnumTypeWrapper): ...

ORDER_STATE_UNSPECIFIED: OrderState.ValueType  # 0
ORDER_STATE_PENDING_CREATED: OrderState.ValueType  # 1
ORDER_STATE_CREATED: OrderState.ValueType  # 2
ORDER_STATE_NEW: OrderState.ValueType  # 3
ORDER_STATE_FILLED: OrderState.ValueType  # 6
ORDER_STATE_ERRORED: OrderState.ValueType  # 10
global___OrderState = OrderState

class _TradingError:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _TradingErrorEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_TradingError.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    TRADING_ERROR_UNSPECIFIED: _TradingError.ValueType  # 0
    TRADING_ERROR_FORBIDDEN: _TradingError.ValueType  # 403
    TRADING_ERROR_NOT_FOUND: _TradingError.ValueType  # 404
    TRADING_ERROR_TRADING_DISABLED: _TradingError.ValueType  # 405
    TRADING_ERROR_TRADING_SUSPENDED: _TradingError.ValueType  # 406
    TRADING_ERROR_NOT_SUBSCRIBED: _TradingError.ValueType  # 407
    TRADING_ERROR_INVALID_SYMBOL: _TradingError.ValueType  # 413
    TRADING_ERROR_INVALID_VALUE: _TradingError.ValueType  # 422
    TRADING_ERROR_INVALID_STATE: _TradingError.ValueType  # 424
    TRADING_ERROR_INVALID_PRICE: _TradingError.ValueType  # 425
    TRADING_ERROR_CONTRACT_EXPIRING: _TradingError.ValueType  # 426
    TRADING_ERROR_INTERNAL_ERROR: _TradingError.ValueType  # 500
    TRADING_ERROR_NOT_IMPLEMENTED: _TradingError.ValueType  # 501

class TradingError(_TradingError, metaclass=_TradingErrorEnumTypeWrapper): ...

TRADING_ERROR_UNSPECIFIED: TradingError.ValueType  # 0
TRADING_ERROR_FORBIDDEN: TradingError.ValueType  # 403
TRADING_ERROR_NOT_FOUND: TradingError.ValueType  # 404
TRADING_ERROR_TRADING_DISABLED: TradingError.ValueType  # 405
TRADING_ERROR_TRADING_SUSPENDED: TradingError.ValueType  # 406
TRADING_ERROR_NOT_SUBSCRIBED: TradingError.ValueType  # 407
TRADING_ERROR_INVALID_SYMBOL: TradingError.ValueType  # 413
TRADING_ERROR_INVALID_VALUE: TradingError.ValueType  # 422
TRADING_ERROR_INVALID_STATE: TradingError.ValueType  # 424
TRADING_ERROR_INVALID_PRICE: TradingError.ValueType  # 425
TRADING_ERROR_CONTRACT_EXPIRING: TradingError.ValueType  # 426
TRADING_ERROR_INTERNAL_ERROR: TradingError.ValueType  # 500
TRADING_ERROR_NOT_IMPLEMENTED: TradingError.ValueType  # 501
global___TradingError = TradingError

class _Channel:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _ChannelEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_Channel.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    CHANNEL_UNSPECIFIED: _Channel.ValueType  # 0
    CHANNEL_ORDERS: _Channel.ValueType  # 1
    CHANNEL_SERVER_INFO: _Channel.ValueType  # 2
    CHANNEL_TICKERS: _Channel.ValueType  # 3
    CHANNEL_RFQ: _Channel.ValueType  # 4
    CHANNEL_ORDER_BOOK_TOP: _Channel.ValueType  # 5

class Channel(_Channel, metaclass=_ChannelEnumTypeWrapper): ...

CHANNEL_UNSPECIFIED: Channel.ValueType  # 0
CHANNEL_ORDERS: Channel.ValueType  # 1
CHANNEL_SERVER_INFO: Channel.ValueType  # 2
CHANNEL_TICKERS: Channel.ValueType  # 3
CHANNEL_RFQ: Channel.ValueType  # 4
CHANNEL_ORDER_BOOK_TOP: Channel.ValueType  # 5
global___Channel = Channel
